------------------------------------------------------------------------------
--                           G E L A   A S I S                              --
--       ASIS implementation for Gela project, a portable Ada compiler      --
--                          http://gela.ada-ru.org                          --
--                     - - - - - - - - - - - - - - -                        --
--            Read copyright and license at the end of this file            --
------------------------------------------------------------------------------
--  $Revision: 209 $ $Date: 2013-11-30 21:03:24 +0200 (Сб, 30 ноя 2013) $
--  Purpose:

with Gela.Encodings; use Gela.Encodings;
with Gela.Decoders.UTF_8;
with Gela.Decoders.Fixed_Width_8; use Gela.Decoders.Fixed_Width_8;

private package Gela.Decoders.Constants is
   subtype W is Wide_Character;
   subtype Fixed_Decoder is Fixed_Width_8.Decoder;

   type Translation_Table_Access is access all Translation_Table;

   ISO_8859_1_Table : aliased Translation_Table :=
     (255 => W'Val (  16#FF#));

   ISO_8859_2_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#104#),
      162 => W'Val ( 16#2D8#), 163 => W'Val ( 16#141#), 164 => W'Val (  16#A4#),
      165 => W'Val ( 16#13D#), 166 => W'Val ( 16#15A#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val ( 16#160#), 170 => W'Val ( 16#15E#),
      171 => W'Val ( 16#164#), 172 => W'Val ( 16#179#), 173 => W'Val (  16#AD#),
      174 => W'Val ( 16#17D#), 175 => W'Val ( 16#17B#), 176 => W'Val (  16#B0#),
      177 => W'Val ( 16#105#), 178 => W'Val ( 16#2DB#), 179 => W'Val ( 16#142#),
      180 => W'Val (  16#B4#), 181 => W'Val ( 16#13E#), 182 => W'Val ( 16#15B#),
      183 => W'Val ( 16#2C7#), 184 => W'Val (  16#B8#), 185 => W'Val ( 16#161#),
      186 => W'Val ( 16#15F#), 187 => W'Val ( 16#165#), 188 => W'Val ( 16#17A#),
      189 => W'Val ( 16#2DD#), 190 => W'Val ( 16#17E#), 191 => W'Val ( 16#17C#),
      192 => W'Val ( 16#154#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val ( 16#102#), 196 => W'Val (  16#C4#), 197 => W'Val ( 16#139#),
      198 => W'Val ( 16#106#), 199 => W'Val (  16#C7#), 200 => W'Val ( 16#10C#),
      201 => W'Val (  16#C9#), 202 => W'Val ( 16#118#), 203 => W'Val (  16#CB#),
      204 => W'Val ( 16#11A#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val ( 16#10E#), 208 => W'Val ( 16#110#), 209 => W'Val ( 16#143#),
      210 => W'Val ( 16#147#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val ( 16#150#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val ( 16#158#), 217 => W'Val ( 16#16E#), 218 => W'Val (  16#DA#),
      219 => W'Val ( 16#170#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val ( 16#162#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#155#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val ( 16#103#),
      228 => W'Val (  16#E4#), 229 => W'Val ( 16#13A#), 230 => W'Val ( 16#107#),
      231 => W'Val (  16#E7#), 232 => W'Val ( 16#10D#), 233 => W'Val (  16#E9#),
      234 => W'Val ( 16#119#), 235 => W'Val (  16#EB#), 236 => W'Val ( 16#11B#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val ( 16#10F#),
      240 => W'Val ( 16#111#), 241 => W'Val ( 16#144#), 242 => W'Val ( 16#148#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val ( 16#151#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val ( 16#159#),
      249 => W'Val ( 16#16F#), 250 => W'Val (  16#FA#), 251 => W'Val ( 16#171#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val ( 16#163#),
      255 => W'Val ( 16#2D9#));

   ISO_8859_3_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#126#),
      162 => W'Val ( 16#2D8#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (  16#A5#), 166 => W'Val ( 16#124#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val ( 16#130#), 170 => W'Val ( 16#15E#),
      171 => W'Val ( 16#11E#), 172 => W'Val ( 16#134#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val ( 16#17B#), 176 => W'Val (  16#B0#),
      177 => W'Val ( 16#127#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val ( 16#125#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val ( 16#131#),
      186 => W'Val ( 16#15F#), 187 => W'Val ( 16#11F#), 188 => W'Val ( 16#135#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val ( 16#17C#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val ( 16#10A#),
      198 => W'Val ( 16#108#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val (  16#D0#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val ( 16#120#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val ( 16#11C#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#16C#),
      222 => W'Val ( 16#15C#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val ( 16#10B#), 230 => W'Val ( 16#109#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val (  16#F0#), 241 => W'Val (  16#F1#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val ( 16#121#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val ( 16#11D#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#16D#), 254 => W'Val ( 16#15D#),
      255 => W'Val ( 16#2D9#));

   ISO_8859_4_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#104#),
      162 => W'Val ( 16#138#), 163 => W'Val ( 16#156#), 164 => W'Val (  16#A4#),
      165 => W'Val ( 16#128#), 166 => W'Val ( 16#13B#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val ( 16#160#), 170 => W'Val ( 16#112#),
      171 => W'Val ( 16#122#), 172 => W'Val ( 16#166#), 173 => W'Val (  16#AD#),
      174 => W'Val ( 16#17D#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val ( 16#105#), 178 => W'Val ( 16#2DB#), 179 => W'Val ( 16#157#),
      180 => W'Val (  16#B4#), 181 => W'Val ( 16#129#), 182 => W'Val ( 16#13C#),
      183 => W'Val ( 16#2C7#), 184 => W'Val (  16#B8#), 185 => W'Val ( 16#161#),
      186 => W'Val ( 16#113#), 187 => W'Val ( 16#123#), 188 => W'Val ( 16#167#),
      189 => W'Val ( 16#14A#), 190 => W'Val ( 16#17E#), 191 => W'Val ( 16#14B#),
      192 => W'Val ( 16#100#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val ( 16#12E#), 200 => W'Val ( 16#10C#),
      201 => W'Val (  16#C9#), 202 => W'Val ( 16#118#), 203 => W'Val (  16#CB#),
      204 => W'Val ( 16#116#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val ( 16#12A#), 208 => W'Val ( 16#110#), 209 => W'Val ( 16#145#),
      210 => W'Val ( 16#14C#), 211 => W'Val ( 16#136#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val ( 16#172#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#168#),
      222 => W'Val ( 16#16A#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#101#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val ( 16#12F#), 232 => W'Val ( 16#10D#), 233 => W'Val (  16#E9#),
      234 => W'Val ( 16#119#), 235 => W'Val (  16#EB#), 236 => W'Val ( 16#117#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val ( 16#12B#),
      240 => W'Val ( 16#111#), 241 => W'Val ( 16#146#), 242 => W'Val ( 16#14D#),
      243 => W'Val ( 16#137#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val ( 16#173#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#169#), 254 => W'Val ( 16#16B#),
      255 => W'Val ( 16#2D9#));

   ISO_8859_5_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#401#),
      162 => W'Val ( 16#402#), 163 => W'Val ( 16#403#), 164 => W'Val ( 16#404#),
      165 => W'Val ( 16#405#), 166 => W'Val ( 16#406#), 167 => W'Val ( 16#407#),
      168 => W'Val ( 16#408#), 169 => W'Val ( 16#409#), 170 => W'Val ( 16#40A#),
      171 => W'Val ( 16#40B#), 172 => W'Val ( 16#40C#), 173 => W'Val (  16#AD#),
      174 => W'Val ( 16#40E#), 175 => W'Val ( 16#40F#), 176 => W'Val ( 16#410#),
      177 => W'Val ( 16#411#), 178 => W'Val ( 16#412#), 179 => W'Val ( 16#413#),
      180 => W'Val ( 16#414#), 181 => W'Val ( 16#415#), 182 => W'Val ( 16#416#),
      183 => W'Val ( 16#417#), 184 => W'Val ( 16#418#), 185 => W'Val ( 16#419#),
      186 => W'Val ( 16#41A#), 187 => W'Val ( 16#41B#), 188 => W'Val ( 16#41C#),
      189 => W'Val ( 16#41D#), 190 => W'Val ( 16#41E#), 191 => W'Val ( 16#41F#),
      192 => W'Val ( 16#420#), 193 => W'Val ( 16#421#), 194 => W'Val ( 16#422#),
      195 => W'Val ( 16#423#), 196 => W'Val ( 16#424#), 197 => W'Val ( 16#425#),
      198 => W'Val ( 16#426#), 199 => W'Val ( 16#427#), 200 => W'Val ( 16#428#),
      201 => W'Val ( 16#429#), 202 => W'Val ( 16#42A#), 203 => W'Val ( 16#42B#),
      204 => W'Val ( 16#42C#), 205 => W'Val ( 16#42D#), 206 => W'Val ( 16#42E#),
      207 => W'Val ( 16#42F#), 208 => W'Val ( 16#430#), 209 => W'Val ( 16#431#),
      210 => W'Val ( 16#432#), 211 => W'Val ( 16#433#), 212 => W'Val ( 16#434#),
      213 => W'Val ( 16#435#), 214 => W'Val ( 16#436#), 215 => W'Val ( 16#437#),
      216 => W'Val ( 16#438#), 217 => W'Val ( 16#439#), 218 => W'Val ( 16#43A#),
      219 => W'Val ( 16#43B#), 220 => W'Val ( 16#43C#), 221 => W'Val ( 16#43D#),
      222 => W'Val ( 16#43E#), 223 => W'Val ( 16#43F#), 224 => W'Val ( 16#440#),
      225 => W'Val ( 16#441#), 226 => W'Val ( 16#442#), 227 => W'Val ( 16#443#),
      228 => W'Val ( 16#444#), 229 => W'Val ( 16#445#), 230 => W'Val ( 16#446#),
      231 => W'Val ( 16#447#), 232 => W'Val ( 16#448#), 233 => W'Val ( 16#449#),
      234 => W'Val ( 16#44A#), 235 => W'Val ( 16#44B#), 236 => W'Val ( 16#44C#),
      237 => W'Val ( 16#44D#), 238 => W'Val ( 16#44E#), 239 => W'Val ( 16#44F#),
      240 => W'Val (16#2116#), 241 => W'Val ( 16#451#), 242 => W'Val ( 16#452#),
      243 => W'Val ( 16#453#), 244 => W'Val ( 16#454#), 245 => W'Val ( 16#455#),
      246 => W'Val ( 16#456#), 247 => W'Val ( 16#457#), 248 => W'Val ( 16#458#),
      249 => W'Val ( 16#459#), 250 => W'Val ( 16#45A#), 251 => W'Val ( 16#45B#),
      252 => W'Val ( 16#45C#), 253 => W'Val (  16#A7#), 254 => W'Val ( 16#45E#),
      255 => W'Val ( 16#45F#));

   ISO_8859_6_Table : aliased Translation_Table :=
     (172 => W'Val ( 16#60C#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#BA#), 187 => W'Val ( 16#61B#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val ( 16#61F#),
      192 => W'Val (  16#C0#), 193 => W'Val ( 16#621#), 194 => W'Val ( 16#622#),
      195 => W'Val ( 16#623#), 196 => W'Val ( 16#624#), 197 => W'Val ( 16#625#),
      198 => W'Val ( 16#626#), 199 => W'Val ( 16#627#), 200 => W'Val ( 16#628#),
      201 => W'Val ( 16#629#), 202 => W'Val ( 16#62A#), 203 => W'Val ( 16#62B#),
      204 => W'Val ( 16#62C#), 205 => W'Val ( 16#62D#), 206 => W'Val ( 16#62E#),
      207 => W'Val ( 16#62F#), 208 => W'Val ( 16#630#), 209 => W'Val ( 16#631#),
      210 => W'Val ( 16#632#), 211 => W'Val ( 16#633#), 212 => W'Val ( 16#634#),
      213 => W'Val ( 16#635#), 214 => W'Val ( 16#636#), 215 => W'Val ( 16#637#),
      216 => W'Val ( 16#638#), 217 => W'Val ( 16#639#), 218 => W'Val ( 16#63A#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val (  16#DE#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#640#),
      225 => W'Val ( 16#641#), 226 => W'Val ( 16#642#), 227 => W'Val ( 16#643#),
      228 => W'Val ( 16#644#), 229 => W'Val ( 16#645#), 230 => W'Val ( 16#646#),
      231 => W'Val ( 16#647#), 232 => W'Val ( 16#648#), 233 => W'Val ( 16#649#),
      234 => W'Val ( 16#64A#), 235 => W'Val ( 16#64B#), 236 => W'Val ( 16#64C#),
      237 => W'Val ( 16#64D#), 238 => W'Val ( 16#64E#), 239 => W'Val ( 16#64F#),
      240 => W'Val ( 16#650#), 241 => W'Val ( 16#651#), 242 => W'Val ( 16#652#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val (  16#FE#),
      255 => W'Val (  16#FF#));

   ISO_8859_7_Table : aliased Translation_Table :=
     (161 => W'Val (16#2018#),
      162 => W'Val (16#2019#), 163 => W'Val (  16#A3#), 164 => W'Val (16#20AC#),
      165 => W'Val (16#20AF#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#37A#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (16#2015#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val ( 16#384#), 181 => W'Val ( 16#385#), 182 => W'Val ( 16#386#),
      183 => W'Val (  16#B7#), 184 => W'Val ( 16#388#), 185 => W'Val ( 16#389#),
      186 => W'Val ( 16#38A#), 187 => W'Val (  16#BB#), 188 => W'Val ( 16#38C#),
      189 => W'Val (  16#BD#), 190 => W'Val ( 16#38E#), 191 => W'Val ( 16#38F#),
      192 => W'Val ( 16#390#), 193 => W'Val ( 16#391#), 194 => W'Val ( 16#392#),
      195 => W'Val ( 16#393#), 196 => W'Val ( 16#394#), 197 => W'Val ( 16#395#),
      198 => W'Val ( 16#396#), 199 => W'Val ( 16#397#), 200 => W'Val ( 16#398#),
      201 => W'Val ( 16#399#), 202 => W'Val ( 16#39A#), 203 => W'Val ( 16#39B#),
      204 => W'Val ( 16#39C#), 205 => W'Val ( 16#39D#), 206 => W'Val ( 16#39E#),
      207 => W'Val ( 16#39F#), 208 => W'Val ( 16#3A0#), 209 => W'Val ( 16#3A1#),
      210 => W'Val (  16#D2#), 211 => W'Val ( 16#3A3#), 212 => W'Val ( 16#3A4#),
      213 => W'Val ( 16#3A5#), 214 => W'Val ( 16#3A6#), 215 => W'Val ( 16#3A7#),
      216 => W'Val ( 16#3A8#), 217 => W'Val ( 16#3A9#), 218 => W'Val ( 16#3AA#),
      219 => W'Val ( 16#3AB#), 220 => W'Val ( 16#3AC#), 221 => W'Val ( 16#3AD#),
      222 => W'Val ( 16#3AE#), 223 => W'Val ( 16#3AF#), 224 => W'Val ( 16#3B0#),
      225 => W'Val ( 16#3B1#), 226 => W'Val ( 16#3B2#), 227 => W'Val ( 16#3B3#),
      228 => W'Val ( 16#3B4#), 229 => W'Val ( 16#3B5#), 230 => W'Val ( 16#3B6#),
      231 => W'Val ( 16#3B7#), 232 => W'Val ( 16#3B8#), 233 => W'Val ( 16#3B9#),
      234 => W'Val ( 16#3BA#), 235 => W'Val ( 16#3BB#), 236 => W'Val ( 16#3BC#),
      237 => W'Val ( 16#3BD#), 238 => W'Val ( 16#3BE#), 239 => W'Val ( 16#3BF#),
      240 => W'Val ( 16#3C0#), 241 => W'Val ( 16#3C1#), 242 => W'Val ( 16#3C2#),
      243 => W'Val ( 16#3C3#), 244 => W'Val ( 16#3C4#), 245 => W'Val ( 16#3C5#),
      246 => W'Val ( 16#3C6#), 247 => W'Val ( 16#3C7#), 248 => W'Val ( 16#3C8#),
      249 => W'Val ( 16#3C9#), 250 => W'Val ( 16#3CA#), 251 => W'Val ( 16#3CB#),
      252 => W'Val ( 16#3CC#), 253 => W'Val ( 16#3CD#), 254 => W'Val ( 16#3CE#),
      255 => W'Val (  16#FF#));

   ISO_8859_8_Table : aliased Translation_Table :=
     (170 => W'Val (  16#D7#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#F7#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#BF#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val (  16#D0#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val (  16#DE#), 223 => W'Val (16#2017#), 224 => W'Val ( 16#5D0#),
      225 => W'Val ( 16#5D1#), 226 => W'Val ( 16#5D2#), 227 => W'Val ( 16#5D3#),
      228 => W'Val ( 16#5D4#), 229 => W'Val ( 16#5D5#), 230 => W'Val ( 16#5D6#),
      231 => W'Val ( 16#5D7#), 232 => W'Val ( 16#5D8#), 233 => W'Val ( 16#5D9#),
      234 => W'Val ( 16#5DA#), 235 => W'Val ( 16#5DB#), 236 => W'Val ( 16#5DC#),
      237 => W'Val ( 16#5DD#), 238 => W'Val ( 16#5DE#), 239 => W'Val ( 16#5DF#),
      240 => W'Val ( 16#5E0#), 241 => W'Val ( 16#5E1#), 242 => W'Val ( 16#5E2#),
      243 => W'Val ( 16#5E3#), 244 => W'Val ( 16#5E4#), 245 => W'Val ( 16#5E5#),
      246 => W'Val ( 16#5E6#), 247 => W'Val ( 16#5E7#), 248 => W'Val ( 16#5E8#),
      249 => W'Val ( 16#5E9#), 250 => W'Val ( 16#5EA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (16#200E#), 254 => W'Val (16#200F#),
      255 => W'Val (  16#FF#));

   ISO_8859_9_Table : aliased Translation_Table :=
     (208 => W'Val ( 16#11E#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#130#),
      222 => W'Val ( 16#15E#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val ( 16#11F#), 241 => W'Val (  16#F1#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#131#), 254 => W'Val ( 16#15F#),
      255 => W'Val (  16#FF#));

   ISO_8859_10_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#104#),
      162 => W'Val ( 16#112#), 163 => W'Val ( 16#122#), 164 => W'Val ( 16#12A#),
      165 => W'Val ( 16#128#), 166 => W'Val ( 16#136#), 167 => W'Val (  16#A7#),
      168 => W'Val ( 16#13B#), 169 => W'Val ( 16#110#), 170 => W'Val ( 16#160#),
      171 => W'Val ( 16#166#), 172 => W'Val ( 16#17D#), 173 => W'Val (  16#AD#),
      174 => W'Val ( 16#16A#), 175 => W'Val ( 16#14A#), 176 => W'Val (  16#B0#),
      177 => W'Val ( 16#105#), 178 => W'Val ( 16#113#), 179 => W'Val ( 16#123#),
      180 => W'Val ( 16#12B#), 181 => W'Val ( 16#129#), 182 => W'Val ( 16#137#),
      183 => W'Val (  16#B7#), 184 => W'Val ( 16#13C#), 185 => W'Val ( 16#111#),
      186 => W'Val ( 16#161#), 187 => W'Val ( 16#167#), 188 => W'Val ( 16#17E#),
      189 => W'Val (16#2015#), 190 => W'Val ( 16#16B#), 191 => W'Val ( 16#14B#),
      192 => W'Val ( 16#100#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val ( 16#12E#), 200 => W'Val ( 16#10C#),
      201 => W'Val (  16#C9#), 202 => W'Val ( 16#118#), 203 => W'Val (  16#CB#),
      204 => W'Val ( 16#116#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val (  16#D0#), 209 => W'Val ( 16#145#),
      210 => W'Val ( 16#14C#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val ( 16#168#),
      216 => W'Val (  16#D8#), 217 => W'Val ( 16#172#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val (  16#DE#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#101#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val ( 16#12F#), 232 => W'Val ( 16#10D#), 233 => W'Val (  16#E9#),
      234 => W'Val ( 16#119#), 235 => W'Val (  16#EB#), 236 => W'Val ( 16#117#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val (  16#F0#), 241 => W'Val ( 16#146#), 242 => W'Val ( 16#14D#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val ( 16#169#), 248 => W'Val (  16#F8#),
      249 => W'Val ( 16#173#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val (  16#FE#),
      255 => W'Val ( 16#138#));

   ISO_8859_11_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#E01#),
      162 => W'Val ( 16#E02#), 163 => W'Val ( 16#E03#), 164 => W'Val ( 16#E04#),
      165 => W'Val ( 16#E05#), 166 => W'Val ( 16#E06#), 167 => W'Val ( 16#E07#),
      168 => W'Val ( 16#E08#), 169 => W'Val ( 16#E09#), 170 => W'Val ( 16#E0A#),
      171 => W'Val ( 16#E0B#), 172 => W'Val ( 16#E0C#), 173 => W'Val ( 16#E0D#),
      174 => W'Val ( 16#E0E#), 175 => W'Val ( 16#E0F#), 176 => W'Val ( 16#E10#),
      177 => W'Val ( 16#E11#), 178 => W'Val ( 16#E12#), 179 => W'Val ( 16#E13#),
      180 => W'Val ( 16#E14#), 181 => W'Val ( 16#E15#), 182 => W'Val ( 16#E16#),
      183 => W'Val ( 16#E17#), 184 => W'Val ( 16#E18#), 185 => W'Val ( 16#E19#),
      186 => W'Val ( 16#E1A#), 187 => W'Val ( 16#E1B#), 188 => W'Val ( 16#E1C#),
      189 => W'Val ( 16#E1D#), 190 => W'Val ( 16#E1E#), 191 => W'Val ( 16#E1F#),
      192 => W'Val ( 16#E20#), 193 => W'Val ( 16#E21#), 194 => W'Val ( 16#E22#),
      195 => W'Val ( 16#E23#), 196 => W'Val ( 16#E24#), 197 => W'Val ( 16#E25#),
      198 => W'Val ( 16#E26#), 199 => W'Val ( 16#E27#), 200 => W'Val ( 16#E28#),
      201 => W'Val ( 16#E29#), 202 => W'Val ( 16#E2A#), 203 => W'Val ( 16#E2B#),
      204 => W'Val ( 16#E2C#), 205 => W'Val ( 16#E2D#), 206 => W'Val ( 16#E2E#),
      207 => W'Val ( 16#E2F#), 208 => W'Val ( 16#E30#), 209 => W'Val ( 16#E31#),
      210 => W'Val ( 16#E32#), 211 => W'Val ( 16#E33#), 212 => W'Val ( 16#E34#),
      213 => W'Val ( 16#E35#), 214 => W'Val ( 16#E36#), 215 => W'Val ( 16#E37#),
      216 => W'Val ( 16#E38#), 217 => W'Val ( 16#E39#), 218 => W'Val ( 16#E3A#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val (  16#DE#), 223 => W'Val ( 16#E3F#), 224 => W'Val ( 16#E40#),
      225 => W'Val ( 16#E41#), 226 => W'Val ( 16#E42#), 227 => W'Val ( 16#E43#),
      228 => W'Val ( 16#E44#), 229 => W'Val ( 16#E45#), 230 => W'Val ( 16#E46#),
      231 => W'Val ( 16#E47#), 232 => W'Val ( 16#E48#), 233 => W'Val ( 16#E49#),
      234 => W'Val ( 16#E4A#), 235 => W'Val ( 16#E4B#), 236 => W'Val ( 16#E4C#),
      237 => W'Val ( 16#E4D#), 238 => W'Val ( 16#E4E#), 239 => W'Val ( 16#E4F#),
      240 => W'Val ( 16#E50#), 241 => W'Val ( 16#E51#), 242 => W'Val ( 16#E52#),
      243 => W'Val ( 16#E53#), 244 => W'Val ( 16#E54#), 245 => W'Val ( 16#E55#),
      246 => W'Val ( 16#E56#), 247 => W'Val ( 16#E57#), 248 => W'Val ( 16#E58#),
      249 => W'Val ( 16#E59#), 250 => W'Val ( 16#E5A#), 251 => W'Val ( 16#E5B#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val (  16#FE#),
      255 => W'Val (  16#FF#));

   ISO_8859_13_Table : aliased Translation_Table :=
     (161 => W'Val (16#201D#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (16#201E#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#D8#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#156#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#C6#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (16#201C#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#F8#), 185 => W'Val (  16#B9#),
      186 => W'Val ( 16#157#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#E6#),
      192 => W'Val ( 16#104#), 193 => W'Val ( 16#12E#), 194 => W'Val ( 16#100#),
      195 => W'Val ( 16#106#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val ( 16#118#), 199 => W'Val ( 16#112#), 200 => W'Val ( 16#10C#),
      201 => W'Val (  16#C9#), 202 => W'Val ( 16#179#), 203 => W'Val ( 16#116#),
      204 => W'Val ( 16#122#), 205 => W'Val ( 16#136#), 206 => W'Val ( 16#12A#),
      207 => W'Val ( 16#13B#), 208 => W'Val ( 16#160#), 209 => W'Val ( 16#143#),
      210 => W'Val ( 16#145#), 211 => W'Val (  16#D3#), 212 => W'Val ( 16#14C#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val ( 16#172#), 217 => W'Val ( 16#141#), 218 => W'Val ( 16#15A#),
      219 => W'Val ( 16#16A#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#17B#),
      222 => W'Val ( 16#17D#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#105#),
      225 => W'Val ( 16#12F#), 226 => W'Val ( 16#101#), 227 => W'Val ( 16#107#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val ( 16#119#),
      231 => W'Val ( 16#113#), 232 => W'Val ( 16#10D#), 233 => W'Val (  16#E9#),
      234 => W'Val ( 16#17A#), 235 => W'Val ( 16#117#), 236 => W'Val ( 16#123#),
      237 => W'Val ( 16#137#), 238 => W'Val ( 16#12B#), 239 => W'Val ( 16#13C#),
      240 => W'Val ( 16#161#), 241 => W'Val ( 16#144#), 242 => W'Val ( 16#146#),
      243 => W'Val (  16#F3#), 244 => W'Val ( 16#14D#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val ( 16#173#),
      249 => W'Val ( 16#142#), 250 => W'Val ( 16#15B#), 251 => W'Val ( 16#16B#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#17C#), 254 => W'Val ( 16#17E#),
      255 => W'Val (16#2019#));

   ISO_8859_14_Table : aliased Translation_Table :=
     (161 => W'Val (16#1E02#),
      162 => W'Val (16#1E03#), 163 => W'Val (  16#A3#), 164 => W'Val ( 16#10A#),
      165 => W'Val ( 16#10B#), 166 => W'Val (16#1E0A#), 167 => W'Val (  16#A7#),
      168 => W'Val (16#1E80#), 169 => W'Val (  16#A9#), 170 => W'Val (16#1E82#),
      171 => W'Val (16#1E0B#), 172 => W'Val (16#1EF2#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val ( 16#178#), 176 => W'Val (16#1E1E#),
      177 => W'Val (16#1E1F#), 178 => W'Val ( 16#120#), 179 => W'Val ( 16#121#),
      180 => W'Val (16#1E40#), 181 => W'Val (16#1E41#), 182 => W'Val (  16#B6#),
      183 => W'Val (16#1E56#), 184 => W'Val (16#1E81#), 185 => W'Val (16#1E57#),
      186 => W'Val (16#1E83#), 187 => W'Val (16#1E60#), 188 => W'Val (16#1EF3#),
      189 => W'Val (16#1E84#), 190 => W'Val (16#1E85#), 191 => W'Val (16#1E61#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val ( 16#174#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (16#1E6A#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val ( 16#176#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val ( 16#175#), 241 => W'Val (  16#F1#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (16#1E6B#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val ( 16#177#),
      255 => W'Val (  16#FF#));

   ISO_8859_15_Table : aliased Translation_Table :=
     (164 => W'Val (16#20AC#),
      165 => W'Val (  16#A5#), 166 => W'Val ( 16#160#), 167 => W'Val (  16#A7#),
      168 => W'Val ( 16#161#), 169 => W'Val (  16#A9#), 170 => W'Val (  16#AA#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val ( 16#17D#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val ( 16#17E#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#BA#), 187 => W'Val (  16#BB#), 188 => W'Val ( 16#152#),
      189 => W'Val ( 16#153#), 190 => W'Val ( 16#178#), 191 => W'Val (  16#BF#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val (  16#D0#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val (  16#DE#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val (  16#F0#), 241 => W'Val (  16#F1#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val (  16#FE#),
      255 => W'Val (  16#FF#));

   ISO_8859_16_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#104#),
      162 => W'Val ( 16#105#), 163 => W'Val ( 16#141#), 164 => W'Val (16#20AC#),
      165 => W'Val (16#201E#), 166 => W'Val ( 16#160#), 167 => W'Val (  16#A7#),
      168 => W'Val ( 16#161#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#218#),
      171 => W'Val (  16#AB#), 172 => W'Val ( 16#179#), 173 => W'Val (  16#AD#),
      174 => W'Val ( 16#17A#), 175 => W'Val ( 16#17B#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val ( 16#10C#), 179 => W'Val ( 16#142#),
      180 => W'Val ( 16#17D#), 181 => W'Val (16#201D#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val ( 16#17E#), 185 => W'Val ( 16#10D#),
      186 => W'Val ( 16#219#), 187 => W'Val (  16#BB#), 188 => W'Val ( 16#152#),
      189 => W'Val ( 16#153#), 190 => W'Val ( 16#178#), 191 => W'Val ( 16#17C#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val ( 16#102#), 196 => W'Val (  16#C4#), 197 => W'Val ( 16#106#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val ( 16#110#), 209 => W'Val ( 16#143#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val ( 16#150#), 214 => W'Val (  16#D6#), 215 => W'Val ( 16#15A#),
      216 => W'Val ( 16#170#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#118#),
      222 => W'Val ( 16#21A#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val ( 16#103#),
      228 => W'Val (  16#E4#), 229 => W'Val ( 16#107#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val ( 16#111#), 241 => W'Val ( 16#144#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val ( 16#151#),
      246 => W'Val (  16#F6#), 247 => W'Val ( 16#15B#), 248 => W'Val ( 16#171#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#119#), 254 => W'Val ( 16#21B#),
      255 => W'Val (  16#FF#));

   CP_037_Table : aliased Translation_Table :=
     (  4 => W'Val (  16#9C#),   5 => W'Val (   16#9#),
        6 => W'Val (  16#86#),   7 => W'Val (  16#7F#),   8 => W'Val (  16#97#),
        9 => W'Val (  16#8D#),  10 => W'Val (  16#8E#),  11 => W'Val (   16#B#),
       12 => W'Val (   16#C#),  13 => W'Val (   16#D#),  14 => W'Val (   16#E#),
       15 => W'Val (   16#F#),  16 => W'Val (  16#10#),  17 => W'Val (  16#11#),
       18 => W'Val (  16#12#),  19 => W'Val (  16#13#),  20 => W'Val (  16#9D#),
       21 => W'Val (  16#85#),  22 => W'Val (   16#8#),  23 => W'Val (  16#87#),
       24 => W'Val (  16#18#),  25 => W'Val (  16#19#),  26 => W'Val (  16#92#),
       27 => W'Val (  16#8F#),  28 => W'Val (  16#1C#),  29 => W'Val (  16#1D#),
       30 => W'Val (  16#1E#),  31 => W'Val (  16#1F#),  32 => W'Val (  16#80#),
       33 => W'Val (  16#81#),  34 => W'Val (  16#82#),  35 => W'Val (  16#83#),
       36 => W'Val (  16#84#),  37 => W'Val (   16#A#),  38 => W'Val (  16#17#),
       39 => W'Val (  16#1B#),  40 => W'Val (  16#88#),  41 => W'Val (  16#89#),
       42 => W'Val (  16#8A#),  43 => W'Val (  16#8B#),  44 => W'Val (  16#8C#),
       45 => W'Val (   16#5#),  46 => W'Val (   16#6#),  47 => W'Val (   16#7#),
       48 => W'Val (  16#90#),  49 => W'Val (  16#91#),  50 => W'Val (  16#16#),
       51 => W'Val (  16#93#),  52 => W'Val (  16#94#),  53 => W'Val (  16#95#),
       54 => W'Val (  16#96#),  55 => W'Val (   16#4#),  56 => W'Val (  16#98#),
       57 => W'Val (  16#99#),  58 => W'Val (  16#9A#),  59 => W'Val (  16#9B#),
       60 => W'Val (  16#14#),  61 => W'Val (  16#15#),  62 => W'Val (  16#9E#),
       63 => W'Val (  16#1A#),  64 => W'Val (  16#20#),  65 => W'Val (  16#A0#),
       66 => W'Val (  16#E2#),  67 => W'Val (  16#E4#),  68 => W'Val (  16#E0#),
       69 => W'Val (  16#E1#),  70 => W'Val (  16#E3#),  71 => W'Val (  16#E5#),
       72 => W'Val (  16#E7#),  73 => W'Val (  16#F1#),  74 => W'Val (  16#A2#),
       75 => W'Val (  16#2E#),  76 => W'Val (  16#3C#),  77 => W'Val (  16#28#),
       78 => W'Val (  16#2B#),  79 => W'Val (  16#7C#),  80 => W'Val (  16#26#),
       81 => W'Val (  16#E9#),  82 => W'Val (  16#EA#),  83 => W'Val (  16#EB#),
       84 => W'Val (  16#E8#),  85 => W'Val (  16#ED#),  86 => W'Val (  16#EE#),
       87 => W'Val (  16#EF#),  88 => W'Val (  16#EC#),  89 => W'Val (  16#DF#),
       90 => W'Val (  16#21#),  91 => W'Val (  16#24#),  92 => W'Val (  16#2A#),
       93 => W'Val (  16#29#),  94 => W'Val (  16#3B#),  95 => W'Val (  16#AC#),
       96 => W'Val (  16#2D#),  97 => W'Val (  16#2F#),  98 => W'Val (  16#C2#),
       99 => W'Val (  16#C4#), 100 => W'Val (  16#C0#), 101 => W'Val (  16#C1#),
      102 => W'Val (  16#C3#), 103 => W'Val (  16#C5#), 104 => W'Val (  16#C7#),
      105 => W'Val (  16#D1#), 106 => W'Val (  16#A6#), 107 => W'Val (  16#2C#),
      108 => W'Val (  16#25#), 109 => W'Val (  16#5F#), 110 => W'Val (  16#3E#),
      111 => W'Val (  16#3F#), 112 => W'Val (  16#F8#), 113 => W'Val (  16#C9#),
      114 => W'Val (  16#CA#), 115 => W'Val (  16#CB#), 116 => W'Val (  16#C8#),
      117 => W'Val (  16#CD#), 118 => W'Val (  16#CE#), 119 => W'Val (  16#CF#),
      120 => W'Val (  16#CC#), 121 => W'Val (  16#60#), 122 => W'Val (  16#3A#),
      123 => W'Val (  16#23#), 124 => W'Val (  16#40#), 125 => W'Val (  16#27#),
      126 => W'Val (  16#3D#), 127 => W'Val (  16#22#), 128 => W'Val (  16#D8#),
      129 => W'Val (  16#61#), 130 => W'Val (  16#62#), 131 => W'Val (  16#63#),
      132 => W'Val (  16#64#), 133 => W'Val (  16#65#), 134 => W'Val (  16#66#),
      135 => W'Val (  16#67#), 136 => W'Val (  16#68#), 137 => W'Val (  16#69#),
      138 => W'Val (  16#AB#), 139 => W'Val (  16#BB#), 140 => W'Val (  16#F0#),
      141 => W'Val (  16#FD#), 142 => W'Val (  16#FE#), 143 => W'Val (  16#B1#),
      144 => W'Val (  16#B0#), 145 => W'Val (  16#6A#), 146 => W'Val (  16#6B#),
      147 => W'Val (  16#6C#), 148 => W'Val (  16#6D#), 149 => W'Val (  16#6E#),
      150 => W'Val (  16#6F#), 151 => W'Val (  16#70#), 152 => W'Val (  16#71#),
      153 => W'Val (  16#72#), 154 => W'Val (  16#AA#), 155 => W'Val (  16#BA#),
      156 => W'Val (  16#E6#), 157 => W'Val (  16#B8#), 158 => W'Val (  16#C6#),
      159 => W'Val (  16#A4#), 160 => W'Val (  16#B5#), 161 => W'Val (  16#7E#),
      162 => W'Val (  16#73#), 163 => W'Val (  16#74#), 164 => W'Val (  16#75#),
      165 => W'Val (  16#76#), 166 => W'Val (  16#77#), 167 => W'Val (  16#78#),
      168 => W'Val (  16#79#), 169 => W'Val (  16#7A#), 170 => W'Val (  16#A1#),
      171 => W'Val (  16#BF#), 172 => W'Val (  16#D0#), 173 => W'Val (  16#DD#),
      174 => W'Val (  16#DE#), 175 => W'Val (  16#AE#), 176 => W'Val (  16#5E#),
      177 => W'Val (  16#A3#), 178 => W'Val (  16#A5#), 179 => W'Val (  16#B7#),
      180 => W'Val (  16#A9#), 181 => W'Val (  16#A7#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#BC#), 184 => W'Val (  16#BD#), 185 => W'Val (  16#BE#),
      186 => W'Val (  16#5B#), 187 => W'Val (  16#5D#), 188 => W'Val (  16#AF#),
      189 => W'Val (  16#A8#), 190 => W'Val (  16#B4#), 191 => W'Val (  16#D7#),
      192 => W'Val (  16#7B#), 193 => W'Val (  16#41#), 194 => W'Val (  16#42#),
      195 => W'Val (  16#43#), 196 => W'Val (  16#44#), 197 => W'Val (  16#45#),
      198 => W'Val (  16#46#), 199 => W'Val (  16#47#), 200 => W'Val (  16#48#),
      201 => W'Val (  16#49#), 202 => W'Val (  16#AD#), 203 => W'Val (  16#F4#),
      204 => W'Val (  16#F6#), 205 => W'Val (  16#F2#), 206 => W'Val (  16#F3#),
      207 => W'Val (  16#F5#), 208 => W'Val (  16#7D#), 209 => W'Val (  16#4A#),
      210 => W'Val (  16#4B#), 211 => W'Val (  16#4C#), 212 => W'Val (  16#4D#),
      213 => W'Val (  16#4E#), 214 => W'Val (  16#4F#), 215 => W'Val (  16#50#),
      216 => W'Val (  16#51#), 217 => W'Val (  16#52#), 218 => W'Val (  16#B9#),
      219 => W'Val (  16#FB#), 220 => W'Val (  16#FC#), 221 => W'Val (  16#F9#),
      222 => W'Val (  16#FA#), 223 => W'Val (  16#FF#), 224 => W'Val (  16#5C#),
      225 => W'Val (  16#F7#), 226 => W'Val (  16#53#), 227 => W'Val (  16#54#),
      228 => W'Val (  16#55#), 229 => W'Val (  16#56#), 230 => W'Val (  16#57#),
      231 => W'Val (  16#58#), 232 => W'Val (  16#59#), 233 => W'Val (  16#5A#),
      234 => W'Val (  16#B2#), 235 => W'Val (  16#D4#), 236 => W'Val (  16#D6#),
      237 => W'Val (  16#D2#), 238 => W'Val (  16#D3#), 239 => W'Val (  16#D5#),
      240 => W'Val (  16#30#), 241 => W'Val (  16#31#), 242 => W'Val (  16#32#),
      243 => W'Val (  16#33#), 244 => W'Val (  16#34#), 245 => W'Val (  16#35#),
      246 => W'Val (  16#36#), 247 => W'Val (  16#37#), 248 => W'Val (  16#38#),
      249 => W'Val (  16#39#), 250 => W'Val (  16#B3#), 251 => W'Val (  16#DB#),
      252 => W'Val (  16#DC#), 253 => W'Val (  16#D9#), 254 => W'Val (  16#DA#),
      255 => W'Val (  16#9F#));

   CP_424_Table : aliased Translation_Table :=
     (  4 => W'Val (  16#9C#),   5 => W'Val (   16#9#),
        6 => W'Val (  16#86#),   7 => W'Val (  16#7F#),   8 => W'Val (  16#97#),
        9 => W'Val (  16#8D#),  10 => W'Val (  16#8E#),  11 => W'Val (   16#B#),
       12 => W'Val (   16#C#),  13 => W'Val (   16#D#),  14 => W'Val (   16#E#),
       15 => W'Val (   16#F#),  16 => W'Val (  16#10#),  17 => W'Val (  16#11#),
       18 => W'Val (  16#12#),  19 => W'Val (  16#13#),  20 => W'Val (  16#9D#),
       21 => W'Val (  16#85#),  22 => W'Val (   16#8#),  23 => W'Val (  16#87#),
       24 => W'Val (  16#18#),  25 => W'Val (  16#19#),  26 => W'Val (  16#92#),
       27 => W'Val (  16#8F#),  28 => W'Val (  16#1C#),  29 => W'Val (  16#1D#),
       30 => W'Val (  16#1E#),  31 => W'Val (  16#1F#),  32 => W'Val (  16#80#),
       33 => W'Val (  16#81#),  34 => W'Val (  16#82#),  35 => W'Val (  16#83#),
       36 => W'Val (  16#84#),  37 => W'Val (   16#A#),  38 => W'Val (  16#17#),
       39 => W'Val (  16#1B#),  40 => W'Val (  16#88#),  41 => W'Val (  16#89#),
       42 => W'Val (  16#8A#),  43 => W'Val (  16#8B#),  44 => W'Val (  16#8C#),
       45 => W'Val (   16#5#),  46 => W'Val (   16#6#),  47 => W'Val (   16#7#),
       48 => W'Val (  16#90#),  49 => W'Val (  16#91#),  50 => W'Val (  16#16#),
       51 => W'Val (  16#93#),  52 => W'Val (  16#94#),  53 => W'Val (  16#95#),
       54 => W'Val (  16#96#),  55 => W'Val (   16#4#),  56 => W'Val (  16#98#),
       57 => W'Val (  16#99#),  58 => W'Val (  16#9A#),  59 => W'Val (  16#9B#),
       60 => W'Val (  16#14#),  61 => W'Val (  16#15#),  62 => W'Val (  16#9E#),
       63 => W'Val (  16#1A#),  64 => W'Val (  16#20#),  65 => W'Val ( 16#5D0#),
       66 => W'Val ( 16#5D1#),  67 => W'Val ( 16#5D2#),  68 => W'Val ( 16#5D3#),
       69 => W'Val ( 16#5D4#),  70 => W'Val ( 16#5D5#),  71 => W'Val ( 16#5D6#),
       72 => W'Val ( 16#5D7#),  73 => W'Val ( 16#5D8#),  74 => W'Val (  16#A2#),
       75 => W'Val (  16#2E#),  76 => W'Val (  16#3C#),  77 => W'Val (  16#28#),
       78 => W'Val (  16#2B#),  79 => W'Val (  16#7C#),  80 => W'Val (  16#26#),
       81 => W'Val ( 16#5D9#),  82 => W'Val ( 16#5DA#),  83 => W'Val ( 16#5DB#),
       84 => W'Val ( 16#5DC#),  85 => W'Val ( 16#5DD#),  86 => W'Val ( 16#5DE#),
       87 => W'Val ( 16#5DF#),  88 => W'Val ( 16#5E0#),  89 => W'Val ( 16#5E1#),
       90 => W'Val (  16#21#),  91 => W'Val (  16#24#),  92 => W'Val (  16#2A#),
       93 => W'Val (  16#29#),  94 => W'Val (  16#3B#),  95 => W'Val (  16#AC#),
       96 => W'Val (  16#2D#),  97 => W'Val (  16#2F#),  98 => W'Val ( 16#5E2#),
       99 => W'Val ( 16#5E3#), 100 => W'Val ( 16#5E4#), 101 => W'Val ( 16#5E5#),
      102 => W'Val ( 16#5E6#), 103 => W'Val ( 16#5E7#), 104 => W'Val ( 16#5E8#),
      105 => W'Val ( 16#5E9#), 106 => W'Val (  16#A6#), 107 => W'Val (  16#2C#),
      108 => W'Val (  16#25#), 109 => W'Val (  16#5F#), 110 => W'Val (  16#3E#),
      111 => W'Val (  16#3F#), 112 => W'Val (16#FFFF#), 113 => W'Val ( 16#5EA#),
      114 => W'Val (16#FFFF#), 115 => W'Val (16#FFFF#), 116 => W'Val (  16#A0#),
      117 => W'Val (16#FFFF#), 118 => W'Val (16#FFFF#), 119 => W'Val (16#FFFF#),
      120 => W'Val (16#2017#), 121 => W'Val (  16#60#), 122 => W'Val (  16#3A#),
      123 => W'Val (  16#23#), 124 => W'Val (  16#40#), 125 => W'Val (  16#27#),
      126 => W'Val (  16#3D#), 127 => W'Val (  16#22#), 128 => W'Val (16#FFFF#),
      129 => W'Val (  16#61#), 130 => W'Val (  16#62#), 131 => W'Val (  16#63#),
      132 => W'Val (  16#64#), 133 => W'Val (  16#65#), 134 => W'Val (  16#66#),
      135 => W'Val (  16#67#), 136 => W'Val (  16#68#), 137 => W'Val (  16#69#),
      138 => W'Val (  16#AB#), 139 => W'Val (  16#BB#), 140 => W'Val (16#FFFF#),
      141 => W'Val (16#FFFF#), 142 => W'Val (16#FFFF#), 143 => W'Val (  16#B1#),
      144 => W'Val (  16#B0#), 145 => W'Val (  16#6A#), 146 => W'Val (  16#6B#),
      147 => W'Val (  16#6C#), 148 => W'Val (  16#6D#), 149 => W'Val (  16#6E#),
      150 => W'Val (  16#6F#), 151 => W'Val (  16#70#), 152 => W'Val (  16#71#),
      153 => W'Val (  16#72#), 154 => W'Val (16#FFFF#), 155 => W'Val (16#FFFF#),
      156 => W'Val (16#FFFF#), 157 => W'Val (  16#B8#), 158 => W'Val (16#FFFF#),
      159 => W'Val (  16#A4#), 160 => W'Val (  16#B5#), 161 => W'Val (  16#7E#),
      162 => W'Val (  16#73#), 163 => W'Val (  16#74#), 164 => W'Val (  16#75#),
      165 => W'Val (  16#76#), 166 => W'Val (  16#77#), 167 => W'Val (  16#78#),
      168 => W'Val (  16#79#), 169 => W'Val (  16#7A#), 170 => W'Val (16#FFFF#),
      171 => W'Val (16#FFFF#), 172 => W'Val (16#FFFF#), 173 => W'Val (16#FFFF#),
      174 => W'Val (16#FFFF#), 175 => W'Val (  16#AE#), 176 => W'Val (  16#5E#),
      177 => W'Val (  16#A3#), 178 => W'Val (  16#A5#), 179 => W'Val (  16#B7#),
      180 => W'Val (  16#A9#), 181 => W'Val (  16#A7#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#BC#), 184 => W'Val (  16#BD#), 185 => W'Val (  16#BE#),
      186 => W'Val (  16#5B#), 187 => W'Val (  16#5D#), 188 => W'Val (  16#AF#),
      189 => W'Val (  16#A8#), 190 => W'Val (  16#B4#), 191 => W'Val (  16#D7#),
      192 => W'Val (  16#7B#), 193 => W'Val (  16#41#), 194 => W'Val (  16#42#),
      195 => W'Val (  16#43#), 196 => W'Val (  16#44#), 197 => W'Val (  16#45#),
      198 => W'Val (  16#46#), 199 => W'Val (  16#47#), 200 => W'Val (  16#48#),
      201 => W'Val (  16#49#), 202 => W'Val (  16#AD#), 203 => W'Val (16#FFFF#),
      204 => W'Val (16#FFFF#), 205 => W'Val (16#FFFF#), 206 => W'Val (16#FFFF#),
      207 => W'Val (16#FFFF#), 208 => W'Val (  16#7D#), 209 => W'Val (  16#4A#),
      210 => W'Val (  16#4B#), 211 => W'Val (  16#4C#), 212 => W'Val (  16#4D#),
      213 => W'Val (  16#4E#), 214 => W'Val (  16#4F#), 215 => W'Val (  16#50#),
      216 => W'Val (  16#51#), 217 => W'Val (  16#52#), 218 => W'Val (  16#B9#),
      219 => W'Val (16#FFFF#), 220 => W'Val (16#FFFF#), 221 => W'Val (16#FFFF#),
      222 => W'Val (16#FFFF#), 223 => W'Val (16#FFFF#), 224 => W'Val (  16#5C#),
      225 => W'Val (  16#F7#), 226 => W'Val (  16#53#), 227 => W'Val (  16#54#),
      228 => W'Val (  16#55#), 229 => W'Val (  16#56#), 230 => W'Val (  16#57#),
      231 => W'Val (  16#58#), 232 => W'Val (  16#59#), 233 => W'Val (  16#5A#),
      234 => W'Val (  16#B2#), 235 => W'Val (16#FFFF#), 236 => W'Val (16#FFFF#),
      237 => W'Val (16#FFFF#), 238 => W'Val (16#FFFF#), 239 => W'Val (16#FFFF#),
      240 => W'Val (  16#30#), 241 => W'Val (  16#31#), 242 => W'Val (  16#32#),
      243 => W'Val (  16#33#), 244 => W'Val (  16#34#), 245 => W'Val (  16#35#),
      246 => W'Val (  16#36#), 247 => W'Val (  16#37#), 248 => W'Val (  16#38#),
      249 => W'Val (  16#39#), 250 => W'Val (  16#B3#), 251 => W'Val (16#FFFF#),
      252 => W'Val (16#FFFF#), 253 => W'Val (16#FFFF#), 254 => W'Val (16#FFFF#),
      255 => W'Val (  16#9F#));

   CP_437_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#E5#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#EF#), 140 => W'Val (  16#EE#),
      141 => W'Val (  16#EC#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val (  16#F2#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#F9#), 152 => W'Val (  16#FF#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#A2#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#A5#), 158 => W'Val (16#20A7#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val (  16#AA#), 167 => W'Val (  16#BA#),
      168 => W'Val (  16#BF#), 169 => W'Val (16#2310#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B1#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#221E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val ( 16#3B5#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_500_Table : aliased Translation_Table :=
     (  4 => W'Val (  16#9C#),   5 => W'Val (   16#9#),
        6 => W'Val (  16#86#),   7 => W'Val (  16#7F#),   8 => W'Val (  16#97#),
        9 => W'Val (  16#8D#),  10 => W'Val (  16#8E#),  11 => W'Val (   16#B#),
       12 => W'Val (   16#C#),  13 => W'Val (   16#D#),  14 => W'Val (   16#E#),
       15 => W'Val (   16#F#),  16 => W'Val (  16#10#),  17 => W'Val (  16#11#),
       18 => W'Val (  16#12#),  19 => W'Val (  16#13#),  20 => W'Val (  16#9D#),
       21 => W'Val (  16#85#),  22 => W'Val (   16#8#),  23 => W'Val (  16#87#),
       24 => W'Val (  16#18#),  25 => W'Val (  16#19#),  26 => W'Val (  16#92#),
       27 => W'Val (  16#8F#),  28 => W'Val (  16#1C#),  29 => W'Val (  16#1D#),
       30 => W'Val (  16#1E#),  31 => W'Val (  16#1F#),  32 => W'Val (  16#80#),
       33 => W'Val (  16#81#),  34 => W'Val (  16#82#),  35 => W'Val (  16#83#),
       36 => W'Val (  16#84#),  37 => W'Val (   16#A#),  38 => W'Val (  16#17#),
       39 => W'Val (  16#1B#),  40 => W'Val (  16#88#),  41 => W'Val (  16#89#),
       42 => W'Val (  16#8A#),  43 => W'Val (  16#8B#),  44 => W'Val (  16#8C#),
       45 => W'Val (   16#5#),  46 => W'Val (   16#6#),  47 => W'Val (   16#7#),
       48 => W'Val (  16#90#),  49 => W'Val (  16#91#),  50 => W'Val (  16#16#),
       51 => W'Val (  16#93#),  52 => W'Val (  16#94#),  53 => W'Val (  16#95#),
       54 => W'Val (  16#96#),  55 => W'Val (   16#4#),  56 => W'Val (  16#98#),
       57 => W'Val (  16#99#),  58 => W'Val (  16#9A#),  59 => W'Val (  16#9B#),
       60 => W'Val (  16#14#),  61 => W'Val (  16#15#),  62 => W'Val (  16#9E#),
       63 => W'Val (  16#1A#),  64 => W'Val (  16#20#),  65 => W'Val (  16#A0#),
       66 => W'Val (  16#E2#),  67 => W'Val (  16#E4#),  68 => W'Val (  16#E0#),
       69 => W'Val (  16#E1#),  70 => W'Val (  16#E3#),  71 => W'Val (  16#E5#),
       72 => W'Val (  16#E7#),  73 => W'Val (  16#F1#),  74 => W'Val (  16#5B#),
       75 => W'Val (  16#2E#),  76 => W'Val (  16#3C#),  77 => W'Val (  16#28#),
       78 => W'Val (  16#2B#),  79 => W'Val (  16#21#),  80 => W'Val (  16#26#),
       81 => W'Val (  16#E9#),  82 => W'Val (  16#EA#),  83 => W'Val (  16#EB#),
       84 => W'Val (  16#E8#),  85 => W'Val (  16#ED#),  86 => W'Val (  16#EE#),
       87 => W'Val (  16#EF#),  88 => W'Val (  16#EC#),  89 => W'Val (  16#DF#),
       90 => W'Val (  16#5D#),  91 => W'Val (  16#24#),  92 => W'Val (  16#2A#),
       93 => W'Val (  16#29#),  94 => W'Val (  16#3B#),  95 => W'Val (  16#5E#),
       96 => W'Val (  16#2D#),  97 => W'Val (  16#2F#),  98 => W'Val (  16#C2#),
       99 => W'Val (  16#C4#), 100 => W'Val (  16#C0#), 101 => W'Val (  16#C1#),
      102 => W'Val (  16#C3#), 103 => W'Val (  16#C5#), 104 => W'Val (  16#C7#),
      105 => W'Val (  16#D1#), 106 => W'Val (  16#A6#), 107 => W'Val (  16#2C#),
      108 => W'Val (  16#25#), 109 => W'Val (  16#5F#), 110 => W'Val (  16#3E#),
      111 => W'Val (  16#3F#), 112 => W'Val (  16#F8#), 113 => W'Val (  16#C9#),
      114 => W'Val (  16#CA#), 115 => W'Val (  16#CB#), 116 => W'Val (  16#C8#),
      117 => W'Val (  16#CD#), 118 => W'Val (  16#CE#), 119 => W'Val (  16#CF#),
      120 => W'Val (  16#CC#), 121 => W'Val (  16#60#), 122 => W'Val (  16#3A#),
      123 => W'Val (  16#23#), 124 => W'Val (  16#40#), 125 => W'Val (  16#27#),
      126 => W'Val (  16#3D#), 127 => W'Val (  16#22#), 128 => W'Val (  16#D8#),
      129 => W'Val (  16#61#), 130 => W'Val (  16#62#), 131 => W'Val (  16#63#),
      132 => W'Val (  16#64#), 133 => W'Val (  16#65#), 134 => W'Val (  16#66#),
      135 => W'Val (  16#67#), 136 => W'Val (  16#68#), 137 => W'Val (  16#69#),
      138 => W'Val (  16#AB#), 139 => W'Val (  16#BB#), 140 => W'Val (  16#F0#),
      141 => W'Val (  16#FD#), 142 => W'Val (  16#FE#), 143 => W'Val (  16#B1#),
      144 => W'Val (  16#B0#), 145 => W'Val (  16#6A#), 146 => W'Val (  16#6B#),
      147 => W'Val (  16#6C#), 148 => W'Val (  16#6D#), 149 => W'Val (  16#6E#),
      150 => W'Val (  16#6F#), 151 => W'Val (  16#70#), 152 => W'Val (  16#71#),
      153 => W'Val (  16#72#), 154 => W'Val (  16#AA#), 155 => W'Val (  16#BA#),
      156 => W'Val (  16#E6#), 157 => W'Val (  16#B8#), 158 => W'Val (  16#C6#),
      159 => W'Val (  16#A4#), 160 => W'Val (  16#B5#), 161 => W'Val (  16#7E#),
      162 => W'Val (  16#73#), 163 => W'Val (  16#74#), 164 => W'Val (  16#75#),
      165 => W'Val (  16#76#), 166 => W'Val (  16#77#), 167 => W'Val (  16#78#),
      168 => W'Val (  16#79#), 169 => W'Val (  16#7A#), 170 => W'Val (  16#A1#),
      171 => W'Val (  16#BF#), 172 => W'Val (  16#D0#), 173 => W'Val (  16#DD#),
      174 => W'Val (  16#DE#), 175 => W'Val (  16#AE#), 176 => W'Val (  16#A2#),
      177 => W'Val (  16#A3#), 178 => W'Val (  16#A5#), 179 => W'Val (  16#B7#),
      180 => W'Val (  16#A9#), 181 => W'Val (  16#A7#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#BC#), 184 => W'Val (  16#BD#), 185 => W'Val (  16#BE#),
      186 => W'Val (  16#AC#), 187 => W'Val (  16#7C#), 188 => W'Val (  16#AF#),
      189 => W'Val (  16#A8#), 190 => W'Val (  16#B4#), 191 => W'Val (  16#D7#),
      192 => W'Val (  16#7B#), 193 => W'Val (  16#41#), 194 => W'Val (  16#42#),
      195 => W'Val (  16#43#), 196 => W'Val (  16#44#), 197 => W'Val (  16#45#),
      198 => W'Val (  16#46#), 199 => W'Val (  16#47#), 200 => W'Val (  16#48#),
      201 => W'Val (  16#49#), 202 => W'Val (  16#AD#), 203 => W'Val (  16#F4#),
      204 => W'Val (  16#F6#), 205 => W'Val (  16#F2#), 206 => W'Val (  16#F3#),
      207 => W'Val (  16#F5#), 208 => W'Val (  16#7D#), 209 => W'Val (  16#4A#),
      210 => W'Val (  16#4B#), 211 => W'Val (  16#4C#), 212 => W'Val (  16#4D#),
      213 => W'Val (  16#4E#), 214 => W'Val (  16#4F#), 215 => W'Val (  16#50#),
      216 => W'Val (  16#51#), 217 => W'Val (  16#52#), 218 => W'Val (  16#B9#),
      219 => W'Val (  16#FB#), 220 => W'Val (  16#FC#), 221 => W'Val (  16#F9#),
      222 => W'Val (  16#FA#), 223 => W'Val (  16#FF#), 224 => W'Val (  16#5C#),
      225 => W'Val (  16#F7#), 226 => W'Val (  16#53#), 227 => W'Val (  16#54#),
      228 => W'Val (  16#55#), 229 => W'Val (  16#56#), 230 => W'Val (  16#57#),
      231 => W'Val (  16#58#), 232 => W'Val (  16#59#), 233 => W'Val (  16#5A#),
      234 => W'Val (  16#B2#), 235 => W'Val (  16#D4#), 236 => W'Val (  16#D6#),
      237 => W'Val (  16#D2#), 238 => W'Val (  16#D3#), 239 => W'Val (  16#D5#),
      240 => W'Val (  16#30#), 241 => W'Val (  16#31#), 242 => W'Val (  16#32#),
      243 => W'Val (  16#33#), 244 => W'Val (  16#34#), 245 => W'Val (  16#35#),
      246 => W'Val (  16#36#), 247 => W'Val (  16#37#), 248 => W'Val (  16#38#),
      249 => W'Val (  16#39#), 250 => W'Val (  16#B3#), 251 => W'Val (  16#DB#),
      252 => W'Val (  16#DC#), 253 => W'Val (  16#D9#), 254 => W'Val (  16#DA#),
      255 => W'Val (  16#9F#));

   CP_875_Table : aliased Translation_Table :=
     (  4 => W'Val (  16#9C#),   5 => W'Val (   16#9#),
        6 => W'Val (  16#86#),   7 => W'Val (  16#7F#),   8 => W'Val (  16#97#),
        9 => W'Val (  16#8D#),  10 => W'Val (  16#8E#),  11 => W'Val (   16#B#),
       12 => W'Val (   16#C#),  13 => W'Val (   16#D#),  14 => W'Val (   16#E#),
       15 => W'Val (   16#F#),  16 => W'Val (  16#10#),  17 => W'Val (  16#11#),
       18 => W'Val (  16#12#),  19 => W'Val (  16#13#),  20 => W'Val (  16#9D#),
       21 => W'Val (  16#85#),  22 => W'Val (   16#8#),  23 => W'Val (  16#87#),
       24 => W'Val (  16#18#),  25 => W'Val (  16#19#),  26 => W'Val (  16#92#),
       27 => W'Val (  16#8F#),  28 => W'Val (  16#1C#),  29 => W'Val (  16#1D#),
       30 => W'Val (  16#1E#),  31 => W'Val (  16#1F#),  32 => W'Val (  16#80#),
       33 => W'Val (  16#81#),  34 => W'Val (  16#82#),  35 => W'Val (  16#83#),
       36 => W'Val (  16#84#),  37 => W'Val (   16#A#),  38 => W'Val (  16#17#),
       39 => W'Val (  16#1B#),  40 => W'Val (  16#88#),  41 => W'Val (  16#89#),
       42 => W'Val (  16#8A#),  43 => W'Val (  16#8B#),  44 => W'Val (  16#8C#),
       45 => W'Val (   16#5#),  46 => W'Val (   16#6#),  47 => W'Val (   16#7#),
       48 => W'Val (  16#90#),  49 => W'Val (  16#91#),  50 => W'Val (  16#16#),
       51 => W'Val (  16#93#),  52 => W'Val (  16#94#),  53 => W'Val (  16#95#),
       54 => W'Val (  16#96#),  55 => W'Val (   16#4#),  56 => W'Val (  16#98#),
       57 => W'Val (  16#99#),  58 => W'Val (  16#9A#),  59 => W'Val (  16#9B#),
       60 => W'Val (  16#14#),  61 => W'Val (  16#15#),  62 => W'Val (  16#9E#),
       63 => W'Val (  16#1A#),  64 => W'Val (  16#20#),  65 => W'Val ( 16#391#),
       66 => W'Val ( 16#392#),  67 => W'Val ( 16#393#),  68 => W'Val ( 16#394#),
       69 => W'Val ( 16#395#),  70 => W'Val ( 16#396#),  71 => W'Val ( 16#397#),
       72 => W'Val ( 16#398#),  73 => W'Val ( 16#399#),  74 => W'Val (  16#5B#),
       75 => W'Val (  16#2E#),  76 => W'Val (  16#3C#),  77 => W'Val (  16#28#),
       78 => W'Val (  16#2B#),  79 => W'Val (  16#21#),  80 => W'Val (  16#26#),
       81 => W'Val ( 16#39A#),  82 => W'Val ( 16#39B#),  83 => W'Val ( 16#39C#),
       84 => W'Val ( 16#39D#),  85 => W'Val ( 16#39E#),  86 => W'Val ( 16#39F#),
       87 => W'Val ( 16#3A0#),  88 => W'Val ( 16#3A1#),  89 => W'Val ( 16#3A3#),
       90 => W'Val (  16#5D#),  91 => W'Val (  16#24#),  92 => W'Val (  16#2A#),
       93 => W'Val (  16#29#),  94 => W'Val (  16#3B#),  95 => W'Val (  16#5E#),
       96 => W'Val (  16#2D#),  97 => W'Val (  16#2F#),  98 => W'Val ( 16#3A4#),
       99 => W'Val ( 16#3A5#), 100 => W'Val ( 16#3A6#), 101 => W'Val ( 16#3A7#),
      102 => W'Val ( 16#3A8#), 103 => W'Val ( 16#3A9#), 104 => W'Val ( 16#3AA#),
      105 => W'Val ( 16#3AB#), 106 => W'Val (  16#7C#), 107 => W'Val (  16#2C#),
      108 => W'Val (  16#25#), 109 => W'Val (  16#5F#), 110 => W'Val (  16#3E#),
      111 => W'Val (  16#3F#), 112 => W'Val (  16#A8#), 113 => W'Val ( 16#386#),
      114 => W'Val ( 16#388#), 115 => W'Val ( 16#389#), 116 => W'Val (  16#A0#),
      117 => W'Val ( 16#38A#), 118 => W'Val ( 16#38C#), 119 => W'Val ( 16#38E#),
      120 => W'Val ( 16#38F#), 121 => W'Val (  16#60#), 122 => W'Val (  16#3A#),
      123 => W'Val (  16#23#), 124 => W'Val (  16#40#), 125 => W'Val (  16#27#),
      126 => W'Val (  16#3D#), 127 => W'Val (  16#22#), 128 => W'Val ( 16#385#),
      129 => W'Val (  16#61#), 130 => W'Val (  16#62#), 131 => W'Val (  16#63#),
      132 => W'Val (  16#64#), 133 => W'Val (  16#65#), 134 => W'Val (  16#66#),
      135 => W'Val (  16#67#), 136 => W'Val (  16#68#), 137 => W'Val (  16#69#),
      138 => W'Val ( 16#3B1#), 139 => W'Val ( 16#3B2#), 140 => W'Val ( 16#3B3#),
      141 => W'Val ( 16#3B4#), 142 => W'Val ( 16#3B5#), 143 => W'Val ( 16#3B6#),
      144 => W'Val (  16#B0#), 145 => W'Val (  16#6A#), 146 => W'Val (  16#6B#),
      147 => W'Val (  16#6C#), 148 => W'Val (  16#6D#), 149 => W'Val (  16#6E#),
      150 => W'Val (  16#6F#), 151 => W'Val (  16#70#), 152 => W'Val (  16#71#),
      153 => W'Val (  16#72#), 154 => W'Val ( 16#3B7#), 155 => W'Val ( 16#3B8#),
      156 => W'Val ( 16#3B9#), 157 => W'Val ( 16#3BA#), 158 => W'Val ( 16#3BB#),
      159 => W'Val ( 16#3BC#), 160 => W'Val (  16#B4#), 161 => W'Val (  16#7E#),
      162 => W'Val (  16#73#), 163 => W'Val (  16#74#), 164 => W'Val (  16#75#),
      165 => W'Val (  16#76#), 166 => W'Val (  16#77#), 167 => W'Val (  16#78#),
      168 => W'Val (  16#79#), 169 => W'Val (  16#7A#), 170 => W'Val ( 16#3BD#),
      171 => W'Val ( 16#3BE#), 172 => W'Val ( 16#3BF#), 173 => W'Val ( 16#3C0#),
      174 => W'Val ( 16#3C1#), 175 => W'Val ( 16#3C3#), 176 => W'Val (  16#A3#),
      177 => W'Val ( 16#3AC#), 178 => W'Val ( 16#3AD#), 179 => W'Val ( 16#3AE#),
      180 => W'Val ( 16#3CA#), 181 => W'Val ( 16#3AF#), 182 => W'Val ( 16#3CC#),
      183 => W'Val ( 16#3CD#), 184 => W'Val ( 16#3CB#), 185 => W'Val ( 16#3CE#),
      186 => W'Val ( 16#3C2#), 187 => W'Val ( 16#3C4#), 188 => W'Val ( 16#3C5#),
      189 => W'Val ( 16#3C6#), 190 => W'Val ( 16#3C7#), 191 => W'Val ( 16#3C8#),
      192 => W'Val (  16#7B#), 193 => W'Val (  16#41#), 194 => W'Val (  16#42#),
      195 => W'Val (  16#43#), 196 => W'Val (  16#44#), 197 => W'Val (  16#45#),
      198 => W'Val (  16#46#), 199 => W'Val (  16#47#), 200 => W'Val (  16#48#),
      201 => W'Val (  16#49#), 202 => W'Val (  16#AD#), 203 => W'Val ( 16#3C9#),
      204 => W'Val ( 16#390#), 205 => W'Val ( 16#3B0#), 206 => W'Val (16#2018#),
      207 => W'Val (16#2015#), 208 => W'Val (  16#7D#), 209 => W'Val (  16#4A#),
      210 => W'Val (  16#4B#), 211 => W'Val (  16#4C#), 212 => W'Val (  16#4D#),
      213 => W'Val (  16#4E#), 214 => W'Val (  16#4F#), 215 => W'Val (  16#50#),
      216 => W'Val (  16#51#), 217 => W'Val (  16#52#), 218 => W'Val (  16#B1#),
      219 => W'Val (  16#BD#), 220 => W'Val (  16#1A#), 221 => W'Val ( 16#387#),
      222 => W'Val (16#2019#), 223 => W'Val (  16#A6#), 224 => W'Val (  16#5C#),
      225 => W'Val (  16#1A#), 226 => W'Val (  16#53#), 227 => W'Val (  16#54#),
      228 => W'Val (  16#55#), 229 => W'Val (  16#56#), 230 => W'Val (  16#57#),
      231 => W'Val (  16#58#), 232 => W'Val (  16#59#), 233 => W'Val (  16#5A#),
      234 => W'Val (  16#B2#), 235 => W'Val (  16#A7#), 236 => W'Val (  16#1A#),
      237 => W'Val (  16#1A#), 238 => W'Val (  16#AB#), 239 => W'Val (  16#AC#),
      240 => W'Val (  16#30#), 241 => W'Val (  16#31#), 242 => W'Val (  16#32#),
      243 => W'Val (  16#33#), 244 => W'Val (  16#34#), 245 => W'Val (  16#35#),
      246 => W'Val (  16#36#), 247 => W'Val (  16#37#), 248 => W'Val (  16#38#),
      249 => W'Val (  16#39#), 250 => W'Val (  16#B3#), 251 => W'Val (  16#A9#),
      252 => W'Val (  16#1A#), 253 => W'Val (  16#1A#), 254 => W'Val (  16#BB#),
      255 => W'Val (  16#9F#));

   CP_737_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#391#),
      129 => W'Val ( 16#392#), 130 => W'Val ( 16#393#), 131 => W'Val ( 16#394#),
      132 => W'Val ( 16#395#), 133 => W'Val ( 16#396#), 134 => W'Val ( 16#397#),
      135 => W'Val ( 16#398#), 136 => W'Val ( 16#399#), 137 => W'Val ( 16#39A#),
      138 => W'Val ( 16#39B#), 139 => W'Val ( 16#39C#), 140 => W'Val ( 16#39D#),
      141 => W'Val ( 16#39E#), 142 => W'Val ( 16#39F#), 143 => W'Val ( 16#3A0#),
      144 => W'Val ( 16#3A1#), 145 => W'Val ( 16#3A3#), 146 => W'Val ( 16#3A4#),
      147 => W'Val ( 16#3A5#), 148 => W'Val ( 16#3A6#), 149 => W'Val ( 16#3A7#),
      150 => W'Val ( 16#3A8#), 151 => W'Val ( 16#3A9#), 152 => W'Val ( 16#3B1#),
      153 => W'Val ( 16#3B2#), 154 => W'Val ( 16#3B3#), 155 => W'Val ( 16#3B4#),
      156 => W'Val ( 16#3B5#), 157 => W'Val ( 16#3B6#), 158 => W'Val ( 16#3B7#),
      159 => W'Val ( 16#3B8#), 160 => W'Val ( 16#3B9#), 161 => W'Val ( 16#3BA#),
      162 => W'Val ( 16#3BB#), 163 => W'Val ( 16#3BC#), 164 => W'Val ( 16#3BD#),
      165 => W'Val ( 16#3BE#), 166 => W'Val ( 16#3BF#), 167 => W'Val ( 16#3C0#),
      168 => W'Val ( 16#3C1#), 169 => W'Val ( 16#3C3#), 170 => W'Val ( 16#3C2#),
      171 => W'Val ( 16#3C4#), 172 => W'Val ( 16#3C5#), 173 => W'Val ( 16#3C6#),
      174 => W'Val ( 16#3C7#), 175 => W'Val ( 16#3C8#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3C9#),
      225 => W'Val ( 16#3AC#), 226 => W'Val ( 16#3AD#), 227 => W'Val ( 16#3AE#),
      228 => W'Val ( 16#3CA#), 229 => W'Val ( 16#3AF#), 230 => W'Val ( 16#3CC#),
      231 => W'Val ( 16#3CD#), 232 => W'Val ( 16#3CB#), 233 => W'Val ( 16#3CE#),
      234 => W'Val ( 16#386#), 235 => W'Val ( 16#388#), 236 => W'Val ( 16#389#),
      237 => W'Val ( 16#38A#), 238 => W'Val ( 16#38C#), 239 => W'Val ( 16#38E#),
      240 => W'Val ( 16#38F#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val ( 16#3AA#), 245 => W'Val ( 16#3AB#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_775_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#106#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val ( 16#101#),
      132 => W'Val (  16#E4#), 133 => W'Val ( 16#123#), 134 => W'Val (  16#E5#),
      135 => W'Val ( 16#107#), 136 => W'Val ( 16#142#), 137 => W'Val ( 16#113#),
      138 => W'Val ( 16#156#), 139 => W'Val ( 16#157#), 140 => W'Val ( 16#12B#),
      141 => W'Val ( 16#179#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val ( 16#14D#), 148 => W'Val (  16#F6#), 149 => W'Val ( 16#122#),
      150 => W'Val (  16#A2#), 151 => W'Val ( 16#15A#), 152 => W'Val ( 16#15B#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#F8#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D8#), 158 => W'Val (  16#D7#),
      159 => W'Val (  16#A4#), 160 => W'Val ( 16#100#), 161 => W'Val ( 16#12A#),
      162 => W'Val (  16#F3#), 163 => W'Val ( 16#17B#), 164 => W'Val ( 16#17C#),
      165 => W'Val ( 16#17A#), 166 => W'Val (16#201D#), 167 => W'Val (  16#A6#),
      168 => W'Val (  16#A9#), 169 => W'Val (  16#AE#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val ( 16#141#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val ( 16#104#), 182 => W'Val ( 16#10C#),
      183 => W'Val ( 16#118#), 184 => W'Val ( 16#116#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val ( 16#12E#), 190 => W'Val ( 16#160#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val ( 16#172#), 199 => W'Val ( 16#16A#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val ( 16#17D#), 208 => W'Val ( 16#105#), 209 => W'Val ( 16#10D#),
      210 => W'Val ( 16#119#), 211 => W'Val ( 16#117#), 212 => W'Val ( 16#12F#),
      213 => W'Val ( 16#161#), 214 => W'Val ( 16#173#), 215 => W'Val ( 16#16B#),
      216 => W'Val ( 16#17E#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val (  16#D3#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#14C#), 227 => W'Val ( 16#143#),
      228 => W'Val (  16#F5#), 229 => W'Val (  16#D5#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#144#), 232 => W'Val ( 16#136#), 233 => W'Val ( 16#137#),
      234 => W'Val ( 16#13B#), 235 => W'Val ( 16#13C#), 236 => W'Val ( 16#146#),
      237 => W'Val ( 16#112#), 238 => W'Val ( 16#145#), 239 => W'Val (16#2019#),
      240 => W'Val (  16#AD#), 241 => W'Val (  16#B1#), 242 => W'Val (16#201C#),
      243 => W'Val (  16#BE#), 244 => W'Val (  16#B6#), 245 => W'Val (  16#A7#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#201E#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (  16#B9#),
      252 => W'Val (  16#B3#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_850_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#E5#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#EF#), 140 => W'Val (  16#EE#),
      141 => W'Val (  16#EC#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val (  16#F2#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#F9#), 152 => W'Val (  16#FF#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#F8#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D8#), 158 => W'Val (  16#D7#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val (  16#AA#), 167 => W'Val (  16#BA#),
      168 => W'Val (  16#BF#), 169 => W'Val (  16#AE#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (  16#C1#), 182 => W'Val (  16#C2#),
      183 => W'Val (  16#C0#), 184 => W'Val (  16#A9#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (  16#A2#), 190 => W'Val (  16#A5#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (  16#E3#), 199 => W'Val (  16#C3#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (  16#A4#), 208 => W'Val (  16#F0#), 209 => W'Val (  16#D0#),
      210 => W'Val (  16#CA#), 211 => W'Val (  16#CB#), 212 => W'Val (  16#C8#),
      213 => W'Val ( 16#131#), 214 => W'Val (  16#CD#), 215 => W'Val (  16#CE#),
      216 => W'Val (  16#CF#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (  16#A6#),
      222 => W'Val (  16#CC#), 223 => W'Val (16#2580#), 224 => W'Val (  16#D3#),
      225 => W'Val (  16#DF#), 226 => W'Val (  16#D4#), 227 => W'Val (  16#D2#),
      228 => W'Val (  16#F5#), 229 => W'Val (  16#D5#), 230 => W'Val (  16#B5#),
      231 => W'Val (  16#FE#), 232 => W'Val (  16#DE#), 233 => W'Val (  16#DA#),
      234 => W'Val (  16#DB#), 235 => W'Val (  16#D9#), 236 => W'Val (  16#FD#),
      237 => W'Val (  16#DD#), 238 => W'Val (  16#AF#), 239 => W'Val (  16#B4#),
      240 => W'Val (  16#AD#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2017#),
      243 => W'Val (  16#BE#), 244 => W'Val (  16#B6#), 245 => W'Val (  16#A7#),
      246 => W'Val (  16#F7#), 247 => W'Val (  16#B8#), 248 => W'Val (  16#B0#),
      249 => W'Val (  16#A8#), 250 => W'Val (  16#B7#), 251 => W'Val (  16#B9#),
      252 => W'Val (  16#B3#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_852_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val ( 16#16F#), 134 => W'Val ( 16#107#),
      135 => W'Val (  16#E7#), 136 => W'Val ( 16#142#), 137 => W'Val (  16#EB#),
      138 => W'Val ( 16#150#), 139 => W'Val ( 16#151#), 140 => W'Val (  16#EE#),
      141 => W'Val ( 16#179#), 142 => W'Val (  16#C4#), 143 => W'Val ( 16#106#),
      144 => W'Val (  16#C9#), 145 => W'Val ( 16#139#), 146 => W'Val ( 16#13A#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val ( 16#13D#),
      150 => W'Val ( 16#13E#), 151 => W'Val ( 16#15A#), 152 => W'Val ( 16#15B#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val ( 16#164#),
      156 => W'Val ( 16#165#), 157 => W'Val ( 16#141#), 158 => W'Val (  16#D7#),
      159 => W'Val ( 16#10D#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val ( 16#104#),
      165 => W'Val ( 16#105#), 166 => W'Val ( 16#17D#), 167 => W'Val ( 16#17E#),
      168 => W'Val ( 16#118#), 169 => W'Val ( 16#119#), 170 => W'Val (  16#AC#),
      171 => W'Val ( 16#17A#), 172 => W'Val ( 16#10C#), 173 => W'Val ( 16#15F#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (  16#C1#), 182 => W'Val (  16#C2#),
      183 => W'Val ( 16#11A#), 184 => W'Val ( 16#15E#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val ( 16#17B#), 190 => W'Val ( 16#17C#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val ( 16#102#), 199 => W'Val ( 16#103#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (  16#A4#), 208 => W'Val ( 16#111#), 209 => W'Val ( 16#110#),
      210 => W'Val ( 16#10E#), 211 => W'Val (  16#CB#), 212 => W'Val ( 16#10F#),
      213 => W'Val ( 16#147#), 214 => W'Val (  16#CD#), 215 => W'Val (  16#CE#),
      216 => W'Val ( 16#11B#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val ( 16#162#),
      222 => W'Val ( 16#16E#), 223 => W'Val (16#2580#), 224 => W'Val (  16#D3#),
      225 => W'Val (  16#DF#), 226 => W'Val (  16#D4#), 227 => W'Val ( 16#143#),
      228 => W'Val ( 16#144#), 229 => W'Val ( 16#148#), 230 => W'Val ( 16#160#),
      231 => W'Val ( 16#161#), 232 => W'Val ( 16#154#), 233 => W'Val (  16#DA#),
      234 => W'Val ( 16#155#), 235 => W'Val ( 16#170#), 236 => W'Val (  16#FD#),
      237 => W'Val (  16#DD#), 238 => W'Val ( 16#163#), 239 => W'Val (  16#B4#),
      240 => W'Val (  16#AD#), 241 => W'Val ( 16#2DD#), 242 => W'Val ( 16#2DB#),
      243 => W'Val ( 16#2C7#), 244 => W'Val ( 16#2D8#), 245 => W'Val (  16#A7#),
      246 => W'Val (  16#F7#), 247 => W'Val (  16#B8#), 248 => W'Val (  16#B0#),
      249 => W'Val (  16#A8#), 250 => W'Val ( 16#2D9#), 251 => W'Val ( 16#171#),
      252 => W'Val ( 16#158#), 253 => W'Val ( 16#159#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_855_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#452#),
      129 => W'Val ( 16#402#), 130 => W'Val ( 16#453#), 131 => W'Val ( 16#403#),
      132 => W'Val ( 16#451#), 133 => W'Val ( 16#401#), 134 => W'Val ( 16#454#),
      135 => W'Val ( 16#404#), 136 => W'Val ( 16#455#), 137 => W'Val ( 16#405#),
      138 => W'Val ( 16#456#), 139 => W'Val ( 16#406#), 140 => W'Val ( 16#457#),
      141 => W'Val ( 16#407#), 142 => W'Val ( 16#458#), 143 => W'Val ( 16#408#),
      144 => W'Val ( 16#459#), 145 => W'Val ( 16#409#), 146 => W'Val ( 16#45A#),
      147 => W'Val ( 16#40A#), 148 => W'Val ( 16#45B#), 149 => W'Val ( 16#40B#),
      150 => W'Val ( 16#45C#), 151 => W'Val ( 16#40C#), 152 => W'Val ( 16#45E#),
      153 => W'Val ( 16#40E#), 154 => W'Val ( 16#45F#), 155 => W'Val ( 16#40F#),
      156 => W'Val ( 16#44E#), 157 => W'Val ( 16#42E#), 158 => W'Val ( 16#44A#),
      159 => W'Val ( 16#42A#), 160 => W'Val ( 16#430#), 161 => W'Val ( 16#410#),
      162 => W'Val ( 16#431#), 163 => W'Val ( 16#411#), 164 => W'Val ( 16#446#),
      165 => W'Val ( 16#426#), 166 => W'Val ( 16#434#), 167 => W'Val ( 16#414#),
      168 => W'Val ( 16#435#), 169 => W'Val ( 16#415#), 170 => W'Val ( 16#444#),
      171 => W'Val ( 16#424#), 172 => W'Val ( 16#433#), 173 => W'Val ( 16#413#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val ( 16#445#), 182 => W'Val ( 16#425#),
      183 => W'Val ( 16#438#), 184 => W'Val ( 16#418#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val ( 16#439#), 190 => W'Val ( 16#419#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val ( 16#43A#), 199 => W'Val ( 16#41A#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (  16#A4#), 208 => W'Val ( 16#43B#), 209 => W'Val ( 16#41B#),
      210 => W'Val ( 16#43C#), 211 => W'Val ( 16#41C#), 212 => W'Val ( 16#43D#),
      213 => W'Val ( 16#41D#), 214 => W'Val ( 16#43E#), 215 => W'Val ( 16#41E#),
      216 => W'Val ( 16#43F#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val ( 16#41F#),
      222 => W'Val ( 16#44F#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#42F#),
      225 => W'Val ( 16#440#), 226 => W'Val ( 16#420#), 227 => W'Val ( 16#441#),
      228 => W'Val ( 16#421#), 229 => W'Val ( 16#442#), 230 => W'Val ( 16#422#),
      231 => W'Val ( 16#443#), 232 => W'Val ( 16#423#), 233 => W'Val ( 16#436#),
      234 => W'Val ( 16#416#), 235 => W'Val ( 16#432#), 236 => W'Val ( 16#412#),
      237 => W'Val ( 16#44C#), 238 => W'Val ( 16#42C#), 239 => W'Val (16#2116#),
      240 => W'Val (  16#AD#), 241 => W'Val ( 16#44B#), 242 => W'Val ( 16#42B#),
      243 => W'Val ( 16#437#), 244 => W'Val ( 16#417#), 245 => W'Val ( 16#448#),
      246 => W'Val ( 16#428#), 247 => W'Val ( 16#44D#), 248 => W'Val ( 16#42D#),
      249 => W'Val ( 16#449#), 250 => W'Val ( 16#429#), 251 => W'Val ( 16#447#),
      252 => W'Val ( 16#427#), 253 => W'Val (  16#A7#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_856_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#5D0#),
      129 => W'Val ( 16#5D1#), 130 => W'Val ( 16#5D2#), 131 => W'Val ( 16#5D3#),
      132 => W'Val ( 16#5D4#), 133 => W'Val ( 16#5D5#), 134 => W'Val ( 16#5D6#),
      135 => W'Val ( 16#5D7#), 136 => W'Val ( 16#5D8#), 137 => W'Val ( 16#5D9#),
      138 => W'Val ( 16#5DA#), 139 => W'Val ( 16#5DB#), 140 => W'Val ( 16#5DC#),
      141 => W'Val ( 16#5DD#), 142 => W'Val ( 16#5DE#), 143 => W'Val ( 16#5DF#),
      144 => W'Val ( 16#5E0#), 145 => W'Val ( 16#5E1#), 146 => W'Val ( 16#5E2#),
      147 => W'Val ( 16#5E3#), 148 => W'Val ( 16#5E4#), 149 => W'Val ( 16#5E5#),
      150 => W'Val ( 16#5E6#), 151 => W'Val ( 16#5E7#), 152 => W'Val ( 16#5E8#),
      153 => W'Val ( 16#5E9#), 154 => W'Val ( 16#5EA#), 155 => W'Val (16#FFFF#),
      156 => W'Val (  16#A3#), 157 => W'Val (16#FFFF#), 158 => W'Val (  16#D7#),
      159 => W'Val (16#FFFF#), 160 => W'Val (16#FFFF#), 161 => W'Val (16#FFFF#),
      162 => W'Val (16#FFFF#), 163 => W'Val (16#FFFF#), 164 => W'Val (16#FFFF#),
      165 => W'Val (16#FFFF#), 166 => W'Val (16#FFFF#), 167 => W'Val (16#FFFF#),
      168 => W'Val (16#FFFF#), 169 => W'Val (  16#AE#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (16#FFFF#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#FFFF#), 182 => W'Val (16#FFFF#),
      183 => W'Val (16#FFFF#), 184 => W'Val (  16#A9#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (  16#A2#), 190 => W'Val (  16#A5#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#FFFF#), 199 => W'Val (16#FFFF#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (  16#A4#), 208 => W'Val (16#FFFF#), 209 => W'Val (16#FFFF#),
      210 => W'Val (16#FFFF#), 211 => W'Val (16#FFFF#), 212 => W'Val (16#FFFF#),
      213 => W'Val (16#FFFF#), 214 => W'Val (16#FFFF#), 215 => W'Val (16#FFFF#),
      216 => W'Val (16#FFFF#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (  16#A6#),
      222 => W'Val (16#FFFF#), 223 => W'Val (16#2580#), 224 => W'Val (16#FFFF#),
      225 => W'Val (16#FFFF#), 226 => W'Val (16#FFFF#), 227 => W'Val (16#FFFF#),
      228 => W'Val (16#FFFF#), 229 => W'Val (16#FFFF#), 230 => W'Val (  16#B5#),
      231 => W'Val (16#FFFF#), 232 => W'Val (16#FFFF#), 233 => W'Val (16#FFFF#),
      234 => W'Val (16#FFFF#), 235 => W'Val (16#FFFF#), 236 => W'Val (16#FFFF#),
      237 => W'Val (16#FFFF#), 238 => W'Val (  16#AF#), 239 => W'Val (  16#B4#),
      240 => W'Val (  16#AD#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2017#),
      243 => W'Val (  16#BE#), 244 => W'Val (  16#B6#), 245 => W'Val (  16#A7#),
      246 => W'Val (  16#F7#), 247 => W'Val (  16#B8#), 248 => W'Val (  16#B0#),
      249 => W'Val (  16#A8#), 250 => W'Val (  16#B7#), 251 => W'Val (  16#B9#),
      252 => W'Val (  16#B3#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_857_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#E5#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#EF#), 140 => W'Val (  16#EE#),
      141 => W'Val ( 16#131#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val (  16#F2#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#F9#), 152 => W'Val ( 16#130#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#F8#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D8#), 158 => W'Val ( 16#15E#),
      159 => W'Val ( 16#15F#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val ( 16#11E#), 167 => W'Val ( 16#11F#),
      168 => W'Val (  16#BF#), 169 => W'Val (  16#AE#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (  16#C1#), 182 => W'Val (  16#C2#),
      183 => W'Val (  16#C0#), 184 => W'Val (  16#A9#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (  16#A2#), 190 => W'Val (  16#A5#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (  16#E3#), 199 => W'Val (  16#C3#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (  16#A4#), 208 => W'Val (  16#BA#), 209 => W'Val (  16#AA#),
      210 => W'Val (  16#CA#), 211 => W'Val (  16#CB#), 212 => W'Val (  16#C8#),
      213 => W'Val (16#FFFF#), 214 => W'Val (  16#CD#), 215 => W'Val (  16#CE#),
      216 => W'Val (  16#CF#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (  16#A6#),
      222 => W'Val (  16#CC#), 223 => W'Val (16#2580#), 224 => W'Val (  16#D3#),
      225 => W'Val (  16#DF#), 226 => W'Val (  16#D4#), 227 => W'Val (  16#D2#),
      228 => W'Val (  16#F5#), 229 => W'Val (  16#D5#), 230 => W'Val (  16#B5#),
      231 => W'Val (16#FFFF#), 232 => W'Val (  16#D7#), 233 => W'Val (  16#DA#),
      234 => W'Val (  16#DB#), 235 => W'Val (  16#D9#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#FF#), 238 => W'Val (  16#AF#), 239 => W'Val (  16#B4#),
      240 => W'Val (  16#AD#), 241 => W'Val (  16#B1#), 242 => W'Val (16#FFFF#),
      243 => W'Val (  16#BE#), 244 => W'Val (  16#B6#), 245 => W'Val (  16#A7#),
      246 => W'Val (  16#F7#), 247 => W'Val (  16#B8#), 248 => W'Val (  16#B0#),
      249 => W'Val (  16#A8#), 250 => W'Val (  16#B7#), 251 => W'Val (  16#B9#),
      252 => W'Val (  16#B3#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_860_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E3#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#C1#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#CA#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#CD#), 140 => W'Val (  16#D4#),
      141 => W'Val (  16#EC#), 142 => W'Val (  16#C3#), 143 => W'Val (  16#C2#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#C0#), 146 => W'Val (  16#C8#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F5#), 149 => W'Val (  16#F2#),
      150 => W'Val (  16#DA#), 151 => W'Val (  16#F9#), 152 => W'Val (  16#CC#),
      153 => W'Val (  16#D5#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#A2#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D9#), 158 => W'Val (16#20A7#),
      159 => W'Val (  16#D3#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val (  16#AA#), 167 => W'Val (  16#BA#),
      168 => W'Val (  16#BF#), 169 => W'Val (  16#D2#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B1#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#221E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val ( 16#3B5#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_861_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#E5#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#D0#), 140 => W'Val (  16#F0#),
      141 => W'Val (  16#DE#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val (  16#FE#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#DD#), 152 => W'Val (  16#FD#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#F8#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D8#), 158 => W'Val (16#20A7#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#C1#),
      165 => W'Val (  16#CD#), 166 => W'Val (  16#D3#), 167 => W'Val (  16#DA#),
      168 => W'Val (  16#BF#), 169 => W'Val (16#2310#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B1#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#221E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val ( 16#3B5#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_862_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#5D0#),
      129 => W'Val ( 16#5D1#), 130 => W'Val ( 16#5D2#), 131 => W'Val ( 16#5D3#),
      132 => W'Val ( 16#5D4#), 133 => W'Val ( 16#5D5#), 134 => W'Val ( 16#5D6#),
      135 => W'Val ( 16#5D7#), 136 => W'Val ( 16#5D8#), 137 => W'Val ( 16#5D9#),
      138 => W'Val ( 16#5DA#), 139 => W'Val ( 16#5DB#), 140 => W'Val ( 16#5DC#),
      141 => W'Val ( 16#5DD#), 142 => W'Val ( 16#5DE#), 143 => W'Val ( 16#5DF#),
      144 => W'Val ( 16#5E0#), 145 => W'Val ( 16#5E1#), 146 => W'Val ( 16#5E2#),
      147 => W'Val ( 16#5E3#), 148 => W'Val ( 16#5E4#), 149 => W'Val ( 16#5E5#),
      150 => W'Val ( 16#5E6#), 151 => W'Val ( 16#5E7#), 152 => W'Val ( 16#5E8#),
      153 => W'Val ( 16#5E9#), 154 => W'Val ( 16#5EA#), 155 => W'Val (  16#A2#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#A5#), 158 => W'Val (16#20A7#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val (  16#AA#), 167 => W'Val (  16#BA#),
      168 => W'Val (  16#BF#), 169 => W'Val (16#2310#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B1#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#221E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val ( 16#3B5#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_863_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#C2#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#B6#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#EF#), 140 => W'Val (  16#EE#),
      141 => W'Val (16#2017#), 142 => W'Val (  16#C0#), 143 => W'Val (  16#A7#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#C8#), 146 => W'Val (  16#CA#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#CB#), 149 => W'Val (  16#CF#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#F9#), 152 => W'Val (  16#A4#),
      153 => W'Val (  16#D4#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#A2#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D9#), 158 => W'Val (  16#DB#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#A6#), 161 => W'Val (  16#B4#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#A8#),
      165 => W'Val (  16#B8#), 166 => W'Val (  16#B3#), 167 => W'Val (  16#AF#),
      168 => W'Val (  16#CE#), 169 => W'Val (16#2310#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#BE#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B1#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#221E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val ( 16#3B5#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_864_Table : aliased Translation_Table :=
     ( 37 => W'Val ( 16#66A#),  38 => W'Val (  16#26#),
       39 => W'Val (  16#27#),  40 => W'Val (  16#28#),  41 => W'Val (  16#29#),
       42 => W'Val (  16#2A#),  43 => W'Val (  16#2B#),  44 => W'Val (  16#2C#),
       45 => W'Val (  16#2D#),  46 => W'Val (  16#2E#),  47 => W'Val (  16#2F#),
       48 => W'Val (  16#30#),  49 => W'Val (  16#31#),  50 => W'Val (  16#32#),
       51 => W'Val (  16#33#),  52 => W'Val (  16#34#),  53 => W'Val (  16#35#),
       54 => W'Val (  16#36#),  55 => W'Val (  16#37#),  56 => W'Val (  16#38#),
       57 => W'Val (  16#39#),  58 => W'Val (  16#3A#),  59 => W'Val (  16#3B#),
       60 => W'Val (  16#3C#),  61 => W'Val (  16#3D#),  62 => W'Val (  16#3E#),
       63 => W'Val (  16#3F#),  64 => W'Val (  16#40#),  65 => W'Val (  16#41#),
       66 => W'Val (  16#42#),  67 => W'Val (  16#43#),  68 => W'Val (  16#44#),
       69 => W'Val (  16#45#),  70 => W'Val (  16#46#),  71 => W'Val (  16#47#),
       72 => W'Val (  16#48#),  73 => W'Val (  16#49#),  74 => W'Val (  16#4A#),
       75 => W'Val (  16#4B#),  76 => W'Val (  16#4C#),  77 => W'Val (  16#4D#),
       78 => W'Val (  16#4E#),  79 => W'Val (  16#4F#),  80 => W'Val (  16#50#),
       81 => W'Val (  16#51#),  82 => W'Val (  16#52#),  83 => W'Val (  16#53#),
       84 => W'Val (  16#54#),  85 => W'Val (  16#55#),  86 => W'Val (  16#56#),
       87 => W'Val (  16#57#),  88 => W'Val (  16#58#),  89 => W'Val (  16#59#),
       90 => W'Val (  16#5A#),  91 => W'Val (  16#5B#),  92 => W'Val (  16#5C#),
       93 => W'Val (  16#5D#),  94 => W'Val (  16#5E#),  95 => W'Val (  16#5F#),
       96 => W'Val (  16#60#),  97 => W'Val (  16#61#),  98 => W'Val (  16#62#),
       99 => W'Val (  16#63#), 100 => W'Val (  16#64#), 101 => W'Val (  16#65#),
      102 => W'Val (  16#66#), 103 => W'Val (  16#67#), 104 => W'Val (  16#68#),
      105 => W'Val (  16#69#), 106 => W'Val (  16#6A#), 107 => W'Val (  16#6B#),
      108 => W'Val (  16#6C#), 109 => W'Val (  16#6D#), 110 => W'Val (  16#6E#),
      111 => W'Val (  16#6F#), 112 => W'Val (  16#70#), 113 => W'Val (  16#71#),
      114 => W'Val (  16#72#), 115 => W'Val (  16#73#), 116 => W'Val (  16#74#),
      117 => W'Val (  16#75#), 118 => W'Val (  16#76#), 119 => W'Val (  16#77#),
      120 => W'Val (  16#78#), 121 => W'Val (  16#79#), 122 => W'Val (  16#7A#),
      123 => W'Val (  16#7B#), 124 => W'Val (  16#7C#), 125 => W'Val (  16#7D#),
      126 => W'Val (  16#7E#), 127 => W'Val (  16#7F#), 128 => W'Val (  16#B0#),
      129 => W'Val (  16#B7#), 130 => W'Val (16#2219#), 131 => W'Val (16#221A#),
      132 => W'Val (16#2592#), 133 => W'Val (16#2500#), 134 => W'Val (16#2502#),
      135 => W'Val (16#253C#), 136 => W'Val (16#2524#), 137 => W'Val (16#252C#),
      138 => W'Val (16#251C#), 139 => W'Val (16#2534#), 140 => W'Val (16#2510#),
      141 => W'Val (16#250C#), 142 => W'Val (16#2514#), 143 => W'Val (16#2518#),
      144 => W'Val ( 16#3B2#), 145 => W'Val (16#221E#), 146 => W'Val ( 16#3C6#),
      147 => W'Val (  16#B1#), 148 => W'Val (  16#BD#), 149 => W'Val (  16#BC#),
      150 => W'Val (16#2248#), 151 => W'Val (  16#AB#), 152 => W'Val (  16#BB#),
      153 => W'Val (16#FEF7#), 154 => W'Val (16#FEF8#), 155 => W'Val (16#FFFF#),
      156 => W'Val (16#FFFF#), 157 => W'Val (16#FEFB#), 158 => W'Val (16#FEFC#),
      159 => W'Val (16#FFFF#), 160 => W'Val (  16#A0#), 161 => W'Val (  16#AD#),
      162 => W'Val (16#FE82#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (16#FE84#), 166 => W'Val (16#FFFF#), 167 => W'Val (16#FFFF#),
      168 => W'Val (16#FE8E#), 169 => W'Val (16#FE8F#), 170 => W'Val (16#FE95#),
      171 => W'Val (16#FE99#), 172 => W'Val ( 16#60C#), 173 => W'Val (16#FE9D#),
      174 => W'Val (16#FEA1#), 175 => W'Val (16#FEA5#), 176 => W'Val ( 16#660#),
      177 => W'Val ( 16#661#), 178 => W'Val ( 16#662#), 179 => W'Val ( 16#663#),
      180 => W'Val ( 16#664#), 181 => W'Val ( 16#665#), 182 => W'Val ( 16#666#),
      183 => W'Val ( 16#667#), 184 => W'Val ( 16#668#), 185 => W'Val ( 16#669#),
      186 => W'Val (16#FED1#), 187 => W'Val ( 16#61B#), 188 => W'Val (16#FEB1#),
      189 => W'Val (16#FEB5#), 190 => W'Val (16#FEB9#), 191 => W'Val ( 16#61F#),
      192 => W'Val (  16#A2#), 193 => W'Val (16#FE80#), 194 => W'Val (16#FE81#),
      195 => W'Val (16#FE83#), 196 => W'Val (16#FE85#), 197 => W'Val (16#FECA#),
      198 => W'Val (16#FE8B#), 199 => W'Val (16#FE8D#), 200 => W'Val (16#FE91#),
      201 => W'Val (16#FE93#), 202 => W'Val (16#FE97#), 203 => W'Val (16#FE9B#),
      204 => W'Val (16#FE9F#), 205 => W'Val (16#FEA3#), 206 => W'Val (16#FEA7#),
      207 => W'Val (16#FEA9#), 208 => W'Val (16#FEAB#), 209 => W'Val (16#FEAD#),
      210 => W'Val (16#FEAF#), 211 => W'Val (16#FEB3#), 212 => W'Val (16#FEB7#),
      213 => W'Val (16#FEBB#), 214 => W'Val (16#FEBF#), 215 => W'Val (16#FEC1#),
      216 => W'Val (16#FEC5#), 217 => W'Val (16#FECB#), 218 => W'Val (16#FECF#),
      219 => W'Val (  16#A6#), 220 => W'Val (  16#AC#), 221 => W'Val (  16#F7#),
      222 => W'Val (  16#D7#), 223 => W'Val (16#FEC9#), 224 => W'Val ( 16#640#),
      225 => W'Val (16#FED3#), 226 => W'Val (16#FED7#), 227 => W'Val (16#FEDB#),
      228 => W'Val (16#FEDF#), 229 => W'Val (16#FEE3#), 230 => W'Val (16#FEE7#),
      231 => W'Val (16#FEEB#), 232 => W'Val (16#FEED#), 233 => W'Val (16#FEEF#),
      234 => W'Val (16#FEF3#), 235 => W'Val (16#FEBD#), 236 => W'Val (16#FECC#),
      237 => W'Val (16#FECE#), 238 => W'Val (16#FECD#), 239 => W'Val (16#FEE1#),
      240 => W'Val (16#FE7D#), 241 => W'Val ( 16#651#), 242 => W'Val (16#FEE5#),
      243 => W'Val (16#FEE9#), 244 => W'Val (16#FEEC#), 245 => W'Val (16#FEF0#),
      246 => W'Val (16#FEF2#), 247 => W'Val (16#FED0#), 248 => W'Val (16#FED5#),
      249 => W'Val (16#FEF5#), 250 => W'Val (16#FEF6#), 251 => W'Val (16#FEDD#),
      252 => W'Val (16#FED9#), 253 => W'Val (16#FEF1#), 254 => W'Val (16#25A0#),
      255 => W'Val (16#FFFF#));

   CP_865_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#E5#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#EF#), 140 => W'Val (  16#EE#),
      141 => W'Val (  16#EC#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val (  16#F2#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#F9#), 152 => W'Val (  16#FF#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#F8#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#D8#), 158 => W'Val (16#20A7#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val (  16#AA#), 167 => W'Val (  16#BA#),
      168 => W'Val (  16#BF#), 169 => W'Val (16#2310#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#A4#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B1#),
      225 => W'Val (  16#DF#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#221E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val ( 16#3B5#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_866_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#410#),
      129 => W'Val ( 16#411#), 130 => W'Val ( 16#412#), 131 => W'Val ( 16#413#),
      132 => W'Val ( 16#414#), 133 => W'Val ( 16#415#), 134 => W'Val ( 16#416#),
      135 => W'Val ( 16#417#), 136 => W'Val ( 16#418#), 137 => W'Val ( 16#419#),
      138 => W'Val ( 16#41A#), 139 => W'Val ( 16#41B#), 140 => W'Val ( 16#41C#),
      141 => W'Val ( 16#41D#), 142 => W'Val ( 16#41E#), 143 => W'Val ( 16#41F#),
      144 => W'Val ( 16#420#), 145 => W'Val ( 16#421#), 146 => W'Val ( 16#422#),
      147 => W'Val ( 16#423#), 148 => W'Val ( 16#424#), 149 => W'Val ( 16#425#),
      150 => W'Val ( 16#426#), 151 => W'Val ( 16#427#), 152 => W'Val ( 16#428#),
      153 => W'Val ( 16#429#), 154 => W'Val ( 16#42A#), 155 => W'Val ( 16#42B#),
      156 => W'Val ( 16#42C#), 157 => W'Val ( 16#42D#), 158 => W'Val ( 16#42E#),
      159 => W'Val ( 16#42F#), 160 => W'Val ( 16#430#), 161 => W'Val ( 16#431#),
      162 => W'Val ( 16#432#), 163 => W'Val ( 16#433#), 164 => W'Val ( 16#434#),
      165 => W'Val ( 16#435#), 166 => W'Val ( 16#436#), 167 => W'Val ( 16#437#),
      168 => W'Val ( 16#438#), 169 => W'Val ( 16#439#), 170 => W'Val ( 16#43A#),
      171 => W'Val ( 16#43B#), 172 => W'Val ( 16#43C#), 173 => W'Val ( 16#43D#),
      174 => W'Val ( 16#43E#), 175 => W'Val ( 16#43F#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val (16#2561#), 182 => W'Val (16#2562#),
      183 => W'Val (16#2556#), 184 => W'Val (16#2555#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val (16#255C#), 190 => W'Val (16#255B#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val (16#255E#), 199 => W'Val (16#255F#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val (16#2567#), 208 => W'Val (16#2568#), 209 => W'Val (16#2564#),
      210 => W'Val (16#2565#), 211 => W'Val (16#2559#), 212 => W'Val (16#2558#),
      213 => W'Val (16#2552#), 214 => W'Val (16#2553#), 215 => W'Val (16#256B#),
      216 => W'Val (16#256A#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val (16#258C#),
      222 => W'Val (16#2590#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#440#),
      225 => W'Val ( 16#441#), 226 => W'Val ( 16#442#), 227 => W'Val ( 16#443#),
      228 => W'Val ( 16#444#), 229 => W'Val ( 16#445#), 230 => W'Val ( 16#446#),
      231 => W'Val ( 16#447#), 232 => W'Val ( 16#448#), 233 => W'Val ( 16#449#),
      234 => W'Val ( 16#44A#), 235 => W'Val ( 16#44B#), 236 => W'Val ( 16#44C#),
      237 => W'Val ( 16#44D#), 238 => W'Val ( 16#44E#), 239 => W'Val ( 16#44F#),
      240 => W'Val ( 16#401#), 241 => W'Val ( 16#451#), 242 => W'Val ( 16#404#),
      243 => W'Val ( 16#454#), 244 => W'Val ( 16#407#), 245 => W'Val ( 16#457#),
      246 => W'Val ( 16#40E#), 247 => W'Val ( 16#45E#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#2116#), 253 => W'Val (  16#A4#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_869_Table : aliased Translation_Table :=
     (128 => W'Val (16#FFFF#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#FFFF#), 131 => W'Val (16#FFFF#),
      132 => W'Val (16#FFFF#), 133 => W'Val (16#FFFF#), 134 => W'Val ( 16#386#),
      135 => W'Val (16#FFFF#), 136 => W'Val (  16#B7#), 137 => W'Val (  16#AC#),
      138 => W'Val (  16#A6#), 139 => W'Val (16#2018#), 140 => W'Val (16#2019#),
      141 => W'Val ( 16#388#), 142 => W'Val (16#2015#), 143 => W'Val ( 16#389#),
      144 => W'Val ( 16#38A#), 145 => W'Val ( 16#3AA#), 146 => W'Val ( 16#38C#),
      147 => W'Val (16#FFFF#), 148 => W'Val (16#FFFF#), 149 => W'Val ( 16#38E#),
      150 => W'Val ( 16#3AB#), 151 => W'Val (  16#A9#), 152 => W'Val ( 16#38F#),
      153 => W'Val (  16#B2#), 154 => W'Val (  16#B3#), 155 => W'Val ( 16#3AC#),
      156 => W'Val (  16#A3#), 157 => W'Val ( 16#3AD#), 158 => W'Val ( 16#3AE#),
      159 => W'Val ( 16#3AF#), 160 => W'Val ( 16#3CA#), 161 => W'Val ( 16#390#),
      162 => W'Val ( 16#3CC#), 163 => W'Val ( 16#3CD#), 164 => W'Val ( 16#391#),
      165 => W'Val ( 16#392#), 166 => W'Val ( 16#393#), 167 => W'Val ( 16#394#),
      168 => W'Val ( 16#395#), 169 => W'Val ( 16#396#), 170 => W'Val ( 16#397#),
      171 => W'Val (  16#BD#), 172 => W'Val ( 16#398#), 173 => W'Val ( 16#399#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (16#2591#),
      177 => W'Val (16#2592#), 178 => W'Val (16#2593#), 179 => W'Val (16#2502#),
      180 => W'Val (16#2524#), 181 => W'Val ( 16#39A#), 182 => W'Val ( 16#39B#),
      183 => W'Val ( 16#39C#), 184 => W'Val ( 16#39D#), 185 => W'Val (16#2563#),
      186 => W'Val (16#2551#), 187 => W'Val (16#2557#), 188 => W'Val (16#255D#),
      189 => W'Val ( 16#39E#), 190 => W'Val ( 16#39F#), 191 => W'Val (16#2510#),
      192 => W'Val (16#2514#), 193 => W'Val (16#2534#), 194 => W'Val (16#252C#),
      195 => W'Val (16#251C#), 196 => W'Val (16#2500#), 197 => W'Val (16#253C#),
      198 => W'Val ( 16#3A0#), 199 => W'Val ( 16#3A1#), 200 => W'Val (16#255A#),
      201 => W'Val (16#2554#), 202 => W'Val (16#2569#), 203 => W'Val (16#2566#),
      204 => W'Val (16#2560#), 205 => W'Val (16#2550#), 206 => W'Val (16#256C#),
      207 => W'Val ( 16#3A3#), 208 => W'Val ( 16#3A4#), 209 => W'Val ( 16#3A5#),
      210 => W'Val ( 16#3A6#), 211 => W'Val ( 16#3A7#), 212 => W'Val ( 16#3A8#),
      213 => W'Val ( 16#3A9#), 214 => W'Val ( 16#3B1#), 215 => W'Val ( 16#3B2#),
      216 => W'Val ( 16#3B3#), 217 => W'Val (16#2518#), 218 => W'Val (16#250C#),
      219 => W'Val (16#2588#), 220 => W'Val (16#2584#), 221 => W'Val ( 16#3B4#),
      222 => W'Val ( 16#3B5#), 223 => W'Val (16#2580#), 224 => W'Val ( 16#3B6#),
      225 => W'Val ( 16#3B7#), 226 => W'Val ( 16#3B8#), 227 => W'Val ( 16#3B9#),
      228 => W'Val ( 16#3BA#), 229 => W'Val ( 16#3BB#), 230 => W'Val ( 16#3BC#),
      231 => W'Val ( 16#3BD#), 232 => W'Val ( 16#3BE#), 233 => W'Val ( 16#3BF#),
      234 => W'Val ( 16#3C0#), 235 => W'Val ( 16#3C1#), 236 => W'Val ( 16#3C3#),
      237 => W'Val ( 16#3C2#), 238 => W'Val ( 16#3C4#), 239 => W'Val ( 16#384#),
      240 => W'Val (  16#AD#), 241 => W'Val (  16#B1#), 242 => W'Val ( 16#3C5#),
      243 => W'Val ( 16#3C6#), 244 => W'Val ( 16#3C7#), 245 => W'Val (  16#A7#),
      246 => W'Val ( 16#3C8#), 247 => W'Val ( 16#385#), 248 => W'Val (  16#B0#),
      249 => W'Val (  16#A8#), 250 => W'Val ( 16#3C9#), 251 => W'Val ( 16#3CB#),
      252 => W'Val ( 16#3B0#), 253 => W'Val ( 16#3CE#), 254 => W'Val (16#25A0#),
      255 => W'Val (  16#A0#));

   CP_874_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#FFFF#), 131 => W'Val (16#FFFF#),
      132 => W'Val (16#FFFF#), 133 => W'Val (16#2026#), 134 => W'Val (16#FFFF#),
      135 => W'Val (16#FFFF#), 136 => W'Val (16#FFFF#), 137 => W'Val (16#FFFF#),
      138 => W'Val (16#FFFF#), 139 => W'Val (16#FFFF#), 140 => W'Val (16#FFFF#),
      141 => W'Val (16#FFFF#), 142 => W'Val (16#FFFF#), 143 => W'Val (16#FFFF#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val (16#FFFF#),
      153 => W'Val (16#FFFF#), 154 => W'Val (16#FFFF#), 155 => W'Val (16#FFFF#),
      156 => W'Val (16#FFFF#), 157 => W'Val (16#FFFF#), 158 => W'Val (16#FFFF#),
      159 => W'Val (16#FFFF#), 160 => W'Val (  16#A0#), 161 => W'Val ( 16#E01#),
      162 => W'Val ( 16#E02#), 163 => W'Val ( 16#E03#), 164 => W'Val ( 16#E04#),
      165 => W'Val ( 16#E05#), 166 => W'Val ( 16#E06#), 167 => W'Val ( 16#E07#),
      168 => W'Val ( 16#E08#), 169 => W'Val ( 16#E09#), 170 => W'Val ( 16#E0A#),
      171 => W'Val ( 16#E0B#), 172 => W'Val ( 16#E0C#), 173 => W'Val ( 16#E0D#),
      174 => W'Val ( 16#E0E#), 175 => W'Val ( 16#E0F#), 176 => W'Val ( 16#E10#),
      177 => W'Val ( 16#E11#), 178 => W'Val ( 16#E12#), 179 => W'Val ( 16#E13#),
      180 => W'Val ( 16#E14#), 181 => W'Val ( 16#E15#), 182 => W'Val ( 16#E16#),
      183 => W'Val ( 16#E17#), 184 => W'Val ( 16#E18#), 185 => W'Val ( 16#E19#),
      186 => W'Val ( 16#E1A#), 187 => W'Val ( 16#E1B#), 188 => W'Val ( 16#E1C#),
      189 => W'Val ( 16#E1D#), 190 => W'Val ( 16#E1E#), 191 => W'Val ( 16#E1F#),
      192 => W'Val ( 16#E20#), 193 => W'Val ( 16#E21#), 194 => W'Val ( 16#E22#),
      195 => W'Val ( 16#E23#), 196 => W'Val ( 16#E24#), 197 => W'Val ( 16#E25#),
      198 => W'Val ( 16#E26#), 199 => W'Val ( 16#E27#), 200 => W'Val ( 16#E28#),
      201 => W'Val ( 16#E29#), 202 => W'Val ( 16#E2A#), 203 => W'Val ( 16#E2B#),
      204 => W'Val ( 16#E2C#), 205 => W'Val ( 16#E2D#), 206 => W'Val ( 16#E2E#),
      207 => W'Val ( 16#E2F#), 208 => W'Val ( 16#E30#), 209 => W'Val ( 16#E31#),
      210 => W'Val ( 16#E32#), 211 => W'Val ( 16#E33#), 212 => W'Val ( 16#E34#),
      213 => W'Val ( 16#E35#), 214 => W'Val ( 16#E36#), 215 => W'Val ( 16#E37#),
      216 => W'Val ( 16#E38#), 217 => W'Val ( 16#E39#), 218 => W'Val ( 16#E3A#),
      219 => W'Val (16#FFFF#), 220 => W'Val (16#FFFF#), 221 => W'Val (16#FFFF#),
      222 => W'Val (16#FFFF#), 223 => W'Val ( 16#E3F#), 224 => W'Val ( 16#E40#),
      225 => W'Val ( 16#E41#), 226 => W'Val ( 16#E42#), 227 => W'Val ( 16#E43#),
      228 => W'Val ( 16#E44#), 229 => W'Val ( 16#E45#), 230 => W'Val ( 16#E46#),
      231 => W'Val ( 16#E47#), 232 => W'Val ( 16#E48#), 233 => W'Val ( 16#E49#),
      234 => W'Val ( 16#E4A#), 235 => W'Val ( 16#E4B#), 236 => W'Val ( 16#E4C#),
      237 => W'Val ( 16#E4D#), 238 => W'Val ( 16#E4E#), 239 => W'Val ( 16#E4F#),
      240 => W'Val ( 16#E50#), 241 => W'Val ( 16#E51#), 242 => W'Val ( 16#E52#),
      243 => W'Val ( 16#E53#), 244 => W'Val ( 16#E54#), 245 => W'Val ( 16#E55#),
      246 => W'Val ( 16#E56#), 247 => W'Val ( 16#E57#), 248 => W'Val ( 16#E58#),
      249 => W'Val ( 16#E59#), 250 => W'Val ( 16#E5A#), 251 => W'Val ( 16#E5B#),
      252 => W'Val (16#FFFF#), 253 => W'Val (16#FFFF#), 254 => W'Val (16#FFFF#),
      255 => W'Val (16#FFFF#));

   CP_1006_Table : aliased Translation_Table :=
     (161 => W'Val ( 16#6F0#),
      162 => W'Val ( 16#6F1#), 163 => W'Val ( 16#6F2#), 164 => W'Val ( 16#6F3#),
      165 => W'Val ( 16#6F4#), 166 => W'Val ( 16#6F5#), 167 => W'Val ( 16#6F6#),
      168 => W'Val ( 16#6F7#), 169 => W'Val ( 16#6F8#), 170 => W'Val ( 16#6F9#),
      171 => W'Val ( 16#60C#), 172 => W'Val ( 16#61B#), 173 => W'Val (  16#AD#),
      174 => W'Val ( 16#61F#), 175 => W'Val (16#FE81#), 176 => W'Val (16#FE8D#),
      177 => W'Val (16#FE8E#), 178 => W'Val (16#FE8E#), 179 => W'Val (16#FE8F#),
      180 => W'Val (16#FE91#), 181 => W'Val (16#FB56#), 182 => W'Val (16#FB58#),
      183 => W'Val (16#FE93#), 184 => W'Val (16#FE95#), 185 => W'Val (16#FE97#),
      186 => W'Val (16#FB66#), 187 => W'Val (16#FB68#), 188 => W'Val (16#FE99#),
      189 => W'Val (16#FE9B#), 190 => W'Val (16#FE9D#), 191 => W'Val (16#FE9F#),
      192 => W'Val (16#FB7A#), 193 => W'Val (16#FB7C#), 194 => W'Val (16#FEA1#),
      195 => W'Val (16#FEA3#), 196 => W'Val (16#FEA5#), 197 => W'Val (16#FEA7#),
      198 => W'Val (16#FEA9#), 199 => W'Val (16#FB84#), 200 => W'Val (16#FEAB#),
      201 => W'Val (16#FEAD#), 202 => W'Val (16#FB8C#), 203 => W'Val (16#FEAF#),
      204 => W'Val (16#FB8A#), 205 => W'Val (16#FEB1#), 206 => W'Val (16#FEB3#),
      207 => W'Val (16#FEB5#), 208 => W'Val (16#FEB7#), 209 => W'Val (16#FEB9#),
      210 => W'Val (16#FEBB#), 211 => W'Val (16#FEBD#), 212 => W'Val (16#FEBF#),
      213 => W'Val (16#FEC1#), 214 => W'Val (16#FEC5#), 215 => W'Val (16#FEC9#),
      216 => W'Val (16#FECA#), 217 => W'Val (16#FECB#), 218 => W'Val (16#FECC#),
      219 => W'Val (16#FECD#), 220 => W'Val (16#FECE#), 221 => W'Val (16#FECF#),
      222 => W'Val (16#FED0#), 223 => W'Val (16#FED1#), 224 => W'Val (16#FED3#),
      225 => W'Val (16#FED5#), 226 => W'Val (16#FED7#), 227 => W'Val (16#FED9#),
      228 => W'Val (16#FEDB#), 229 => W'Val (16#FB92#), 230 => W'Val (16#FB94#),
      231 => W'Val (16#FEDD#), 232 => W'Val (16#FEDF#), 233 => W'Val (16#FEE0#),
      234 => W'Val (16#FEE1#), 235 => W'Val (16#FEE3#), 236 => W'Val (16#FB9E#),
      237 => W'Val (16#FEE5#), 238 => W'Val (16#FEE7#), 239 => W'Val (16#FE85#),
      240 => W'Val (16#FEED#), 241 => W'Val (16#FBA6#), 242 => W'Val (16#FBA8#),
      243 => W'Val (16#FBA9#), 244 => W'Val (16#FBAA#), 245 => W'Val (16#FE80#),
      246 => W'Val (16#FE89#), 247 => W'Val (16#FE8A#), 248 => W'Val (16#FE8B#),
      249 => W'Val (16#FEF1#), 250 => W'Val (16#FEF2#), 251 => W'Val (16#FEF3#),
      252 => W'Val (16#FBB0#), 253 => W'Val (16#FBAE#), 254 => W'Val (16#FE7C#),
      255 => W'Val (16#FE7D#));

   CP_1026_Table : aliased Translation_Table :=
     (  4 => W'Val (  16#9C#),   5 => W'Val (   16#9#),
        6 => W'Val (  16#86#),   7 => W'Val (  16#7F#),   8 => W'Val (  16#97#),
        9 => W'Val (  16#8D#),  10 => W'Val (  16#8E#),  11 => W'Val (   16#B#),
       12 => W'Val (   16#C#),  13 => W'Val (   16#D#),  14 => W'Val (   16#E#),
       15 => W'Val (   16#F#),  16 => W'Val (  16#10#),  17 => W'Val (  16#11#),
       18 => W'Val (  16#12#),  19 => W'Val (  16#13#),  20 => W'Val (  16#9D#),
       21 => W'Val (  16#85#),  22 => W'Val (   16#8#),  23 => W'Val (  16#87#),
       24 => W'Val (  16#18#),  25 => W'Val (  16#19#),  26 => W'Val (  16#92#),
       27 => W'Val (  16#8F#),  28 => W'Val (  16#1C#),  29 => W'Val (  16#1D#),
       30 => W'Val (  16#1E#),  31 => W'Val (  16#1F#),  32 => W'Val (  16#80#),
       33 => W'Val (  16#81#),  34 => W'Val (  16#82#),  35 => W'Val (  16#83#),
       36 => W'Val (  16#84#),  37 => W'Val (   16#A#),  38 => W'Val (  16#17#),
       39 => W'Val (  16#1B#),  40 => W'Val (  16#88#),  41 => W'Val (  16#89#),
       42 => W'Val (  16#8A#),  43 => W'Val (  16#8B#),  44 => W'Val (  16#8C#),
       45 => W'Val (   16#5#),  46 => W'Val (   16#6#),  47 => W'Val (   16#7#),
       48 => W'Val (  16#90#),  49 => W'Val (  16#91#),  50 => W'Val (  16#16#),
       51 => W'Val (  16#93#),  52 => W'Val (  16#94#),  53 => W'Val (  16#95#),
       54 => W'Val (  16#96#),  55 => W'Val (   16#4#),  56 => W'Val (  16#98#),
       57 => W'Val (  16#99#),  58 => W'Val (  16#9A#),  59 => W'Val (  16#9B#),
       60 => W'Val (  16#14#),  61 => W'Val (  16#15#),  62 => W'Val (  16#9E#),
       63 => W'Val (  16#1A#),  64 => W'Val (  16#20#),  65 => W'Val (  16#A0#),
       66 => W'Val (  16#E2#),  67 => W'Val (  16#E4#),  68 => W'Val (  16#E0#),
       69 => W'Val (  16#E1#),  70 => W'Val (  16#E3#),  71 => W'Val (  16#E5#),
       72 => W'Val (  16#7B#),  73 => W'Val (  16#F1#),  74 => W'Val (  16#C7#),
       75 => W'Val (  16#2E#),  76 => W'Val (  16#3C#),  77 => W'Val (  16#28#),
       78 => W'Val (  16#2B#),  79 => W'Val (  16#21#),  80 => W'Val (  16#26#),
       81 => W'Val (  16#E9#),  82 => W'Val (  16#EA#),  83 => W'Val (  16#EB#),
       84 => W'Val (  16#E8#),  85 => W'Val (  16#ED#),  86 => W'Val (  16#EE#),
       87 => W'Val (  16#EF#),  88 => W'Val (  16#EC#),  89 => W'Val (  16#DF#),
       90 => W'Val ( 16#11E#),  91 => W'Val ( 16#130#),  92 => W'Val (  16#2A#),
       93 => W'Val (  16#29#),  94 => W'Val (  16#3B#),  95 => W'Val (  16#5E#),
       96 => W'Val (  16#2D#),  97 => W'Val (  16#2F#),  98 => W'Val (  16#C2#),
       99 => W'Val (  16#C4#), 100 => W'Val (  16#C0#), 101 => W'Val (  16#C1#),
      102 => W'Val (  16#C3#), 103 => W'Val (  16#C5#), 104 => W'Val (  16#5B#),
      105 => W'Val (  16#D1#), 106 => W'Val ( 16#15F#), 107 => W'Val (  16#2C#),
      108 => W'Val (  16#25#), 109 => W'Val (  16#5F#), 110 => W'Val (  16#3E#),
      111 => W'Val (  16#3F#), 112 => W'Val (  16#F8#), 113 => W'Val (  16#C9#),
      114 => W'Val (  16#CA#), 115 => W'Val (  16#CB#), 116 => W'Val (  16#C8#),
      117 => W'Val (  16#CD#), 118 => W'Val (  16#CE#), 119 => W'Val (  16#CF#),
      120 => W'Val (  16#CC#), 121 => W'Val ( 16#131#), 122 => W'Val (  16#3A#),
      123 => W'Val (  16#D6#), 124 => W'Val ( 16#15E#), 125 => W'Val (  16#27#),
      126 => W'Val (  16#3D#), 127 => W'Val (  16#DC#), 128 => W'Val (  16#D8#),
      129 => W'Val (  16#61#), 130 => W'Val (  16#62#), 131 => W'Val (  16#63#),
      132 => W'Val (  16#64#), 133 => W'Val (  16#65#), 134 => W'Val (  16#66#),
      135 => W'Val (  16#67#), 136 => W'Val (  16#68#), 137 => W'Val (  16#69#),
      138 => W'Val (  16#AB#), 139 => W'Val (  16#BB#), 140 => W'Val (  16#7D#),
      141 => W'Val (  16#60#), 142 => W'Val (  16#A6#), 143 => W'Val (  16#B1#),
      144 => W'Val (  16#B0#), 145 => W'Val (  16#6A#), 146 => W'Val (  16#6B#),
      147 => W'Val (  16#6C#), 148 => W'Val (  16#6D#), 149 => W'Val (  16#6E#),
      150 => W'Val (  16#6F#), 151 => W'Val (  16#70#), 152 => W'Val (  16#71#),
      153 => W'Val (  16#72#), 154 => W'Val (  16#AA#), 155 => W'Val (  16#BA#),
      156 => W'Val (  16#E6#), 157 => W'Val (  16#B8#), 158 => W'Val (  16#C6#),
      159 => W'Val (  16#A4#), 160 => W'Val (  16#B5#), 161 => W'Val (  16#F6#),
      162 => W'Val (  16#73#), 163 => W'Val (  16#74#), 164 => W'Val (  16#75#),
      165 => W'Val (  16#76#), 166 => W'Val (  16#77#), 167 => W'Val (  16#78#),
      168 => W'Val (  16#79#), 169 => W'Val (  16#7A#), 170 => W'Val (  16#A1#),
      171 => W'Val (  16#BF#), 172 => W'Val (  16#5D#), 173 => W'Val (  16#24#),
      174 => W'Val (  16#40#), 175 => W'Val (  16#AE#), 176 => W'Val (  16#A2#),
      177 => W'Val (  16#A3#), 178 => W'Val (  16#A5#), 179 => W'Val (  16#B7#),
      180 => W'Val (  16#A9#), 181 => W'Val (  16#A7#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#BC#), 184 => W'Val (  16#BD#), 185 => W'Val (  16#BE#),
      186 => W'Val (  16#AC#), 187 => W'Val (  16#7C#), 188 => W'Val (  16#AF#),
      189 => W'Val (  16#A8#), 190 => W'Val (  16#B4#), 191 => W'Val (  16#D7#),
      192 => W'Val (  16#E7#), 193 => W'Val (  16#41#), 194 => W'Val (  16#42#),
      195 => W'Val (  16#43#), 196 => W'Val (  16#44#), 197 => W'Val (  16#45#),
      198 => W'Val (  16#46#), 199 => W'Val (  16#47#), 200 => W'Val (  16#48#),
      201 => W'Val (  16#49#), 202 => W'Val (  16#AD#), 203 => W'Val (  16#F4#),
      204 => W'Val (  16#7E#), 205 => W'Val (  16#F2#), 206 => W'Val (  16#F3#),
      207 => W'Val (  16#F5#), 208 => W'Val ( 16#11F#), 209 => W'Val (  16#4A#),
      210 => W'Val (  16#4B#), 211 => W'Val (  16#4C#), 212 => W'Val (  16#4D#),
      213 => W'Val (  16#4E#), 214 => W'Val (  16#4F#), 215 => W'Val (  16#50#),
      216 => W'Val (  16#51#), 217 => W'Val (  16#52#), 218 => W'Val (  16#B9#),
      219 => W'Val (  16#FB#), 220 => W'Val (  16#5C#), 221 => W'Val (  16#F9#),
      222 => W'Val (  16#FA#), 223 => W'Val (  16#FF#), 224 => W'Val (  16#FC#),
      225 => W'Val (  16#F7#), 226 => W'Val (  16#53#), 227 => W'Val (  16#54#),
      228 => W'Val (  16#55#), 229 => W'Val (  16#56#), 230 => W'Val (  16#57#),
      231 => W'Val (  16#58#), 232 => W'Val (  16#59#), 233 => W'Val (  16#5A#),
      234 => W'Val (  16#B2#), 235 => W'Val (  16#D4#), 236 => W'Val (  16#23#),
      237 => W'Val (  16#D2#), 238 => W'Val (  16#D3#), 239 => W'Val (  16#D5#),
      240 => W'Val (  16#30#), 241 => W'Val (  16#31#), 242 => W'Val (  16#32#),
      243 => W'Val (  16#33#), 244 => W'Val (  16#34#), 245 => W'Val (  16#35#),
      246 => W'Val (  16#36#), 247 => W'Val (  16#37#), 248 => W'Val (  16#38#),
      249 => W'Val (  16#39#), 250 => W'Val (  16#B3#), 251 => W'Val (  16#DB#),
      252 => W'Val (  16#22#), 253 => W'Val (  16#D9#), 254 => W'Val (  16#DA#),
      255 => W'Val (  16#9F#));

   CP_1250_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val (16#FFFF#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val (16#FFFF#), 137 => W'Val (16#2030#),
      138 => W'Val ( 16#160#), 139 => W'Val (16#2039#), 140 => W'Val ( 16#15A#),
      141 => W'Val ( 16#164#), 142 => W'Val ( 16#17D#), 143 => W'Val ( 16#179#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val (16#FFFF#),
      153 => W'Val (16#2122#), 154 => W'Val ( 16#161#), 155 => W'Val (16#203A#),
      156 => W'Val ( 16#15B#), 157 => W'Val ( 16#165#), 158 => W'Val ( 16#17E#),
      159 => W'Val ( 16#17A#), 160 => W'Val (  16#A0#), 161 => W'Val ( 16#2C7#),
      162 => W'Val ( 16#2D8#), 163 => W'Val ( 16#141#), 164 => W'Val (  16#A4#),
      165 => W'Val ( 16#104#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#15E#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val ( 16#17B#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val ( 16#2DB#), 179 => W'Val ( 16#142#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val ( 16#105#),
      186 => W'Val ( 16#15F#), 187 => W'Val (  16#BB#), 188 => W'Val ( 16#13D#),
      189 => W'Val ( 16#2DD#), 190 => W'Val ( 16#13E#), 191 => W'Val ( 16#17C#),
      192 => W'Val ( 16#154#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val ( 16#102#), 196 => W'Val (  16#C4#), 197 => W'Val ( 16#139#),
      198 => W'Val ( 16#106#), 199 => W'Val (  16#C7#), 200 => W'Val ( 16#10C#),
      201 => W'Val (  16#C9#), 202 => W'Val ( 16#118#), 203 => W'Val (  16#CB#),
      204 => W'Val ( 16#11A#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val ( 16#10E#), 208 => W'Val ( 16#110#), 209 => W'Val ( 16#143#),
      210 => W'Val ( 16#147#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val ( 16#150#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val ( 16#158#), 217 => W'Val ( 16#16E#), 218 => W'Val (  16#DA#),
      219 => W'Val ( 16#170#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val ( 16#162#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#155#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val ( 16#103#),
      228 => W'Val (  16#E4#), 229 => W'Val ( 16#13A#), 230 => W'Val ( 16#107#),
      231 => W'Val (  16#E7#), 232 => W'Val ( 16#10D#), 233 => W'Val (  16#E9#),
      234 => W'Val ( 16#119#), 235 => W'Val (  16#EB#), 236 => W'Val ( 16#11B#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val ( 16#10F#),
      240 => W'Val ( 16#111#), 241 => W'Val ( 16#144#), 242 => W'Val ( 16#148#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val ( 16#151#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val ( 16#159#),
      249 => W'Val ( 16#16F#), 250 => W'Val (  16#FA#), 251 => W'Val ( 16#171#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val ( 16#163#),
      255 => W'Val ( 16#2D9#));

   CP_1251_Table : aliased Translation_Table :=
     (128 => W'Val ( 16#402#),
      129 => W'Val ( 16#403#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#453#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val (16#20AC#), 137 => W'Val (16#2030#),
      138 => W'Val ( 16#409#), 139 => W'Val (16#2039#), 140 => W'Val ( 16#40A#),
      141 => W'Val ( 16#40C#), 142 => W'Val ( 16#40B#), 143 => W'Val ( 16#40F#),
      144 => W'Val ( 16#452#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val (16#FFFF#),
      153 => W'Val (16#2122#), 154 => W'Val ( 16#459#), 155 => W'Val (16#203A#),
      156 => W'Val ( 16#45A#), 157 => W'Val ( 16#45C#), 158 => W'Val ( 16#45B#),
      159 => W'Val ( 16#45F#), 160 => W'Val (  16#A0#), 161 => W'Val ( 16#40E#),
      162 => W'Val ( 16#45E#), 163 => W'Val ( 16#408#), 164 => W'Val (  16#A4#),
      165 => W'Val ( 16#490#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val ( 16#401#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#404#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val ( 16#407#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val ( 16#406#), 179 => W'Val ( 16#456#),
      180 => W'Val ( 16#491#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val ( 16#451#), 185 => W'Val (16#2116#),
      186 => W'Val ( 16#454#), 187 => W'Val (  16#BB#), 188 => W'Val ( 16#458#),
      189 => W'Val ( 16#405#), 190 => W'Val ( 16#455#), 191 => W'Val ( 16#457#),
      192 => W'Val ( 16#410#), 193 => W'Val ( 16#411#), 194 => W'Val ( 16#412#),
      195 => W'Val ( 16#413#), 196 => W'Val ( 16#414#), 197 => W'Val ( 16#415#),
      198 => W'Val ( 16#416#), 199 => W'Val ( 16#417#), 200 => W'Val ( 16#418#),
      201 => W'Val ( 16#419#), 202 => W'Val ( 16#41A#), 203 => W'Val ( 16#41B#),
      204 => W'Val ( 16#41C#), 205 => W'Val ( 16#41D#), 206 => W'Val ( 16#41E#),
      207 => W'Val ( 16#41F#), 208 => W'Val ( 16#420#), 209 => W'Val ( 16#421#),
      210 => W'Val ( 16#422#), 211 => W'Val ( 16#423#), 212 => W'Val ( 16#424#),
      213 => W'Val ( 16#425#), 214 => W'Val ( 16#426#), 215 => W'Val ( 16#427#),
      216 => W'Val ( 16#428#), 217 => W'Val ( 16#429#), 218 => W'Val ( 16#42A#),
      219 => W'Val ( 16#42B#), 220 => W'Val ( 16#42C#), 221 => W'Val ( 16#42D#),
      222 => W'Val ( 16#42E#), 223 => W'Val ( 16#42F#), 224 => W'Val ( 16#430#),
      225 => W'Val ( 16#431#), 226 => W'Val ( 16#432#), 227 => W'Val ( 16#433#),
      228 => W'Val ( 16#434#), 229 => W'Val ( 16#435#), 230 => W'Val ( 16#436#),
      231 => W'Val ( 16#437#), 232 => W'Val ( 16#438#), 233 => W'Val ( 16#439#),
      234 => W'Val ( 16#43A#), 235 => W'Val ( 16#43B#), 236 => W'Val ( 16#43C#),
      237 => W'Val ( 16#43D#), 238 => W'Val ( 16#43E#), 239 => W'Val ( 16#43F#),
      240 => W'Val ( 16#440#), 241 => W'Val ( 16#441#), 242 => W'Val ( 16#442#),
      243 => W'Val ( 16#443#), 244 => W'Val ( 16#444#), 245 => W'Val ( 16#445#),
      246 => W'Val ( 16#446#), 247 => W'Val ( 16#447#), 248 => W'Val ( 16#448#),
      249 => W'Val ( 16#449#), 250 => W'Val ( 16#44A#), 251 => W'Val ( 16#44B#),
      252 => W'Val ( 16#44C#), 253 => W'Val ( 16#44D#), 254 => W'Val ( 16#44E#),
      255 => W'Val ( 16#44F#));

   CP_1252_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#192#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val ( 16#2C6#), 137 => W'Val (16#2030#),
      138 => W'Val ( 16#160#), 139 => W'Val (16#2039#), 140 => W'Val ( 16#152#),
      141 => W'Val (16#FFFF#), 142 => W'Val ( 16#17D#), 143 => W'Val (16#FFFF#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val ( 16#2DC#),
      153 => W'Val (16#2122#), 154 => W'Val ( 16#161#), 155 => W'Val (16#203A#),
      156 => W'Val ( 16#153#), 157 => W'Val (16#FFFF#), 158 => W'Val ( 16#17E#),
      159 => W'Val ( 16#178#), 160 => W'Val (  16#A0#), 161 => W'Val (  16#A1#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (  16#A5#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val (  16#AA#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#BA#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#BF#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val (  16#D0#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val (  16#DD#),
      222 => W'Val (  16#DE#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val (  16#F0#), 241 => W'Val (  16#F1#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (  16#FD#), 254 => W'Val (  16#FE#),
      255 => W'Val (  16#FF#));

   CP_1253_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#192#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val (16#FFFF#), 137 => W'Val (16#2030#),
      138 => W'Val (16#FFFF#), 139 => W'Val (16#2039#), 140 => W'Val (16#FFFF#),
      141 => W'Val (16#FFFF#), 142 => W'Val (16#FFFF#), 143 => W'Val (16#FFFF#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val (16#FFFF#),
      153 => W'Val (16#2122#), 154 => W'Val (16#FFFF#), 155 => W'Val (16#203A#),
      156 => W'Val (16#FFFF#), 157 => W'Val (16#FFFF#), 158 => W'Val (16#FFFF#),
      159 => W'Val (16#FFFF#), 160 => W'Val (  16#A0#), 161 => W'Val ( 16#385#),
      162 => W'Val ( 16#386#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (  16#A5#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val (16#FFFF#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (16#2015#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val ( 16#384#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val ( 16#388#), 185 => W'Val ( 16#389#),
      186 => W'Val ( 16#38A#), 187 => W'Val (  16#BB#), 188 => W'Val ( 16#38C#),
      189 => W'Val (  16#BD#), 190 => W'Val ( 16#38E#), 191 => W'Val ( 16#38F#),
      192 => W'Val ( 16#390#), 193 => W'Val ( 16#391#), 194 => W'Val ( 16#392#),
      195 => W'Val ( 16#393#), 196 => W'Val ( 16#394#), 197 => W'Val ( 16#395#),
      198 => W'Val ( 16#396#), 199 => W'Val ( 16#397#), 200 => W'Val ( 16#398#),
      201 => W'Val ( 16#399#), 202 => W'Val ( 16#39A#), 203 => W'Val ( 16#39B#),
      204 => W'Val ( 16#39C#), 205 => W'Val ( 16#39D#), 206 => W'Val ( 16#39E#),
      207 => W'Val ( 16#39F#), 208 => W'Val ( 16#3A0#), 209 => W'Val ( 16#3A1#),
      210 => W'Val (16#FFFF#), 211 => W'Val ( 16#3A3#), 212 => W'Val ( 16#3A4#),
      213 => W'Val ( 16#3A5#), 214 => W'Val ( 16#3A6#), 215 => W'Val ( 16#3A7#),
      216 => W'Val ( 16#3A8#), 217 => W'Val ( 16#3A9#), 218 => W'Val ( 16#3AA#),
      219 => W'Val ( 16#3AB#), 220 => W'Val ( 16#3AC#), 221 => W'Val ( 16#3AD#),
      222 => W'Val ( 16#3AE#), 223 => W'Val ( 16#3AF#), 224 => W'Val ( 16#3B0#),
      225 => W'Val ( 16#3B1#), 226 => W'Val ( 16#3B2#), 227 => W'Val ( 16#3B3#),
      228 => W'Val ( 16#3B4#), 229 => W'Val ( 16#3B5#), 230 => W'Val ( 16#3B6#),
      231 => W'Val ( 16#3B7#), 232 => W'Val ( 16#3B8#), 233 => W'Val ( 16#3B9#),
      234 => W'Val ( 16#3BA#), 235 => W'Val ( 16#3BB#), 236 => W'Val ( 16#3BC#),
      237 => W'Val ( 16#3BD#), 238 => W'Val ( 16#3BE#), 239 => W'Val ( 16#3BF#),
      240 => W'Val ( 16#3C0#), 241 => W'Val ( 16#3C1#), 242 => W'Val ( 16#3C2#),
      243 => W'Val ( 16#3C3#), 244 => W'Val ( 16#3C4#), 245 => W'Val ( 16#3C5#),
      246 => W'Val ( 16#3C6#), 247 => W'Val ( 16#3C7#), 248 => W'Val ( 16#3C8#),
      249 => W'Val ( 16#3C9#), 250 => W'Val ( 16#3CA#), 251 => W'Val ( 16#3CB#),
      252 => W'Val ( 16#3CC#), 253 => W'Val ( 16#3CD#), 254 => W'Val ( 16#3CE#),
      255 => W'Val (16#FFFF#));

   CP_1254_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#192#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val ( 16#2C6#), 137 => W'Val (16#2030#),
      138 => W'Val ( 16#160#), 139 => W'Val (16#2039#), 140 => W'Val ( 16#152#),
      141 => W'Val (16#FFFF#), 142 => W'Val (16#FFFF#), 143 => W'Val (16#FFFF#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val ( 16#2DC#),
      153 => W'Val (16#2122#), 154 => W'Val ( 16#161#), 155 => W'Val (16#203A#),
      156 => W'Val ( 16#153#), 157 => W'Val (16#FFFF#), 158 => W'Val (16#FFFF#),
      159 => W'Val ( 16#178#), 160 => W'Val (  16#A0#), 161 => W'Val (  16#A1#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (  16#A5#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val (  16#AA#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#BA#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#BF#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val (  16#C3#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val (  16#CC#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val ( 16#11E#), 209 => W'Val (  16#D1#),
      210 => W'Val (  16#D2#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#130#),
      222 => W'Val ( 16#15E#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val (  16#E3#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val (  16#EC#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val ( 16#11F#), 241 => W'Val (  16#F1#), 242 => W'Val (  16#F2#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#131#), 254 => W'Val ( 16#15F#),
      255 => W'Val (  16#FF#));

   CP_1255_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#192#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val ( 16#2C6#), 137 => W'Val (16#2030#),
      138 => W'Val (16#FFFF#), 139 => W'Val (16#2039#), 140 => W'Val (16#FFFF#),
      141 => W'Val (16#FFFF#), 142 => W'Val (16#FFFF#), 143 => W'Val (16#FFFF#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val ( 16#2DC#),
      153 => W'Val (16#2122#), 154 => W'Val (16#FFFF#), 155 => W'Val (16#203A#),
      156 => W'Val (16#FFFF#), 157 => W'Val (16#FFFF#), 158 => W'Val (16#FFFF#),
      159 => W'Val (16#FFFF#), 160 => W'Val (  16#A0#), 161 => W'Val (  16#A1#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (16#20AA#),
      165 => W'Val (  16#A5#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val (  16#D7#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#F7#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#BF#),
      192 => W'Val ( 16#5B0#), 193 => W'Val ( 16#5B1#), 194 => W'Val ( 16#5B2#),
      195 => W'Val ( 16#5B3#), 196 => W'Val ( 16#5B4#), 197 => W'Val ( 16#5B5#),
      198 => W'Val ( 16#5B6#), 199 => W'Val ( 16#5B7#), 200 => W'Val ( 16#5B8#),
      201 => W'Val ( 16#5B9#), 202 => W'Val (16#FFFF#), 203 => W'Val ( 16#5BB#),
      204 => W'Val ( 16#5BC#), 205 => W'Val ( 16#5BD#), 206 => W'Val ( 16#5BE#),
      207 => W'Val ( 16#5BF#), 208 => W'Val ( 16#5C0#), 209 => W'Val ( 16#5C1#),
      210 => W'Val ( 16#5C2#), 211 => W'Val ( 16#5C3#), 212 => W'Val ( 16#5F0#),
      213 => W'Val ( 16#5F1#), 214 => W'Val ( 16#5F2#), 215 => W'Val ( 16#5F3#),
      216 => W'Val ( 16#5F4#), 217 => W'Val (16#FFFF#), 218 => W'Val (16#FFFF#),
      219 => W'Val (16#FFFF#), 220 => W'Val (16#FFFF#), 221 => W'Val (16#FFFF#),
      222 => W'Val (16#FFFF#), 223 => W'Val (16#FFFF#), 224 => W'Val ( 16#5D0#),
      225 => W'Val ( 16#5D1#), 226 => W'Val ( 16#5D2#), 227 => W'Val ( 16#5D3#),
      228 => W'Val ( 16#5D4#), 229 => W'Val ( 16#5D5#), 230 => W'Val ( 16#5D6#),
      231 => W'Val ( 16#5D7#), 232 => W'Val ( 16#5D8#), 233 => W'Val ( 16#5D9#),
      234 => W'Val ( 16#5DA#), 235 => W'Val ( 16#5DB#), 236 => W'Val ( 16#5DC#),
      237 => W'Val ( 16#5DD#), 238 => W'Val ( 16#5DE#), 239 => W'Val ( 16#5DF#),
      240 => W'Val ( 16#5E0#), 241 => W'Val ( 16#5E1#), 242 => W'Val ( 16#5E2#),
      243 => W'Val ( 16#5E3#), 244 => W'Val ( 16#5E4#), 245 => W'Val ( 16#5E5#),
      246 => W'Val ( 16#5E6#), 247 => W'Val ( 16#5E7#), 248 => W'Val ( 16#5E8#),
      249 => W'Val ( 16#5E9#), 250 => W'Val ( 16#5EA#), 251 => W'Val (16#FFFF#),
      252 => W'Val (16#FFFF#), 253 => W'Val (16#200E#), 254 => W'Val (16#200F#),
      255 => W'Val (16#FFFF#));

   CP_1256_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val ( 16#67E#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#192#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val ( 16#2C6#), 137 => W'Val (16#2030#),
      138 => W'Val ( 16#679#), 139 => W'Val (16#2039#), 140 => W'Val ( 16#152#),
      141 => W'Val ( 16#686#), 142 => W'Val ( 16#698#), 143 => W'Val ( 16#688#),
      144 => W'Val ( 16#6AF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val ( 16#6A9#),
      153 => W'Val (16#2122#), 154 => W'Val ( 16#691#), 155 => W'Val (16#203A#),
      156 => W'Val ( 16#153#), 157 => W'Val (16#200C#), 158 => W'Val (16#200D#),
      159 => W'Val ( 16#6BA#), 160 => W'Val (  16#A0#), 161 => W'Val ( 16#60C#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (  16#A5#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#6BE#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val ( 16#61B#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val ( 16#61F#),
      192 => W'Val ( 16#6C1#), 193 => W'Val ( 16#621#), 194 => W'Val ( 16#622#),
      195 => W'Val ( 16#623#), 196 => W'Val ( 16#624#), 197 => W'Val ( 16#625#),
      198 => W'Val ( 16#626#), 199 => W'Val ( 16#627#), 200 => W'Val ( 16#628#),
      201 => W'Val ( 16#629#), 202 => W'Val ( 16#62A#), 203 => W'Val ( 16#62B#),
      204 => W'Val ( 16#62C#), 205 => W'Val ( 16#62D#), 206 => W'Val ( 16#62E#),
      207 => W'Val ( 16#62F#), 208 => W'Val ( 16#630#), 209 => W'Val ( 16#631#),
      210 => W'Val ( 16#632#), 211 => W'Val ( 16#633#), 212 => W'Val ( 16#634#),
      213 => W'Val ( 16#635#), 214 => W'Val ( 16#636#), 215 => W'Val (  16#D7#),
      216 => W'Val ( 16#637#), 217 => W'Val ( 16#638#), 218 => W'Val ( 16#639#),
      219 => W'Val ( 16#63A#), 220 => W'Val ( 16#640#), 221 => W'Val ( 16#641#),
      222 => W'Val ( 16#642#), 223 => W'Val ( 16#643#), 224 => W'Val (  16#E0#),
      225 => W'Val ( 16#644#), 226 => W'Val (  16#E2#), 227 => W'Val ( 16#645#),
      228 => W'Val ( 16#646#), 229 => W'Val ( 16#647#), 230 => W'Val ( 16#648#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val ( 16#649#),
      237 => W'Val ( 16#64A#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val ( 16#64B#), 241 => W'Val ( 16#64C#), 242 => W'Val ( 16#64D#),
      243 => W'Val ( 16#64E#), 244 => W'Val (  16#F4#), 245 => W'Val ( 16#64F#),
      246 => W'Val ( 16#650#), 247 => W'Val (  16#F7#), 248 => W'Val ( 16#651#),
      249 => W'Val (  16#F9#), 250 => W'Val ( 16#652#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val (16#200E#), 254 => W'Val (16#200F#),
      255 => W'Val ( 16#6D2#));

   CP_1257_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val (16#FFFF#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val (16#FFFF#), 137 => W'Val (16#2030#),
      138 => W'Val (16#FFFF#), 139 => W'Val (16#2039#), 140 => W'Val (16#FFFF#),
      141 => W'Val (  16#A8#), 142 => W'Val ( 16#2C7#), 143 => W'Val (  16#B8#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val (16#FFFF#),
      153 => W'Val (16#2122#), 154 => W'Val (16#FFFF#), 155 => W'Val (16#203A#),
      156 => W'Val (16#FFFF#), 157 => W'Val (  16#AF#), 158 => W'Val ( 16#2DB#),
      159 => W'Val (16#FFFF#), 160 => W'Val (  16#A0#), 161 => W'Val (16#FFFF#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (16#FFFF#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#D8#), 169 => W'Val (  16#A9#), 170 => W'Val ( 16#156#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#C6#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#F8#), 185 => W'Val (  16#B9#),
      186 => W'Val ( 16#157#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#E6#),
      192 => W'Val ( 16#104#), 193 => W'Val ( 16#12E#), 194 => W'Val ( 16#100#),
      195 => W'Val ( 16#106#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val ( 16#118#), 199 => W'Val ( 16#112#), 200 => W'Val ( 16#10C#),
      201 => W'Val (  16#C9#), 202 => W'Val ( 16#179#), 203 => W'Val ( 16#116#),
      204 => W'Val ( 16#122#), 205 => W'Val ( 16#136#), 206 => W'Val ( 16#12A#),
      207 => W'Val ( 16#13B#), 208 => W'Val ( 16#160#), 209 => W'Val ( 16#143#),
      210 => W'Val ( 16#145#), 211 => W'Val (  16#D3#), 212 => W'Val ( 16#14C#),
      213 => W'Val (  16#D5#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val ( 16#172#), 217 => W'Val ( 16#141#), 218 => W'Val ( 16#15A#),
      219 => W'Val ( 16#16A#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#17B#),
      222 => W'Val ( 16#17D#), 223 => W'Val (  16#DF#), 224 => W'Val ( 16#105#),
      225 => W'Val ( 16#12F#), 226 => W'Val ( 16#101#), 227 => W'Val ( 16#107#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val ( 16#119#),
      231 => W'Val ( 16#113#), 232 => W'Val ( 16#10D#), 233 => W'Val (  16#E9#),
      234 => W'Val ( 16#17A#), 235 => W'Val ( 16#117#), 236 => W'Val ( 16#123#),
      237 => W'Val ( 16#137#), 238 => W'Val ( 16#12B#), 239 => W'Val ( 16#13C#),
      240 => W'Val ( 16#161#), 241 => W'Val ( 16#144#), 242 => W'Val ( 16#146#),
      243 => W'Val (  16#F3#), 244 => W'Val ( 16#14D#), 245 => W'Val (  16#F5#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val ( 16#173#),
      249 => W'Val ( 16#142#), 250 => W'Val ( 16#15B#), 251 => W'Val ( 16#16B#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#17C#), 254 => W'Val ( 16#17E#),
      255 => W'Val ( 16#2D9#));

   CP_1258_Table : aliased Translation_Table :=
     (128 => W'Val (16#20AC#),
      129 => W'Val (16#FFFF#), 130 => W'Val (16#201A#), 131 => W'Val ( 16#192#),
      132 => W'Val (16#201E#), 133 => W'Val (16#2026#), 134 => W'Val (16#2020#),
      135 => W'Val (16#2021#), 136 => W'Val ( 16#2C6#), 137 => W'Val (16#2030#),
      138 => W'Val (16#FFFF#), 139 => W'Val (16#2039#), 140 => W'Val ( 16#152#),
      141 => W'Val (16#FFFF#), 142 => W'Val (16#FFFF#), 143 => W'Val (16#FFFF#),
      144 => W'Val (16#FFFF#), 145 => W'Val (16#2018#), 146 => W'Val (16#2019#),
      147 => W'Val (16#201C#), 148 => W'Val (16#201D#), 149 => W'Val (16#2022#),
      150 => W'Val (16#2013#), 151 => W'Val (16#2014#), 152 => W'Val ( 16#2DC#),
      153 => W'Val (16#2122#), 154 => W'Val (16#FFFF#), 155 => W'Val (16#203A#),
      156 => W'Val ( 16#153#), 157 => W'Val (16#FFFF#), 158 => W'Val (16#FFFF#),
      159 => W'Val ( 16#178#), 160 => W'Val (  16#A0#), 161 => W'Val (  16#A1#),
      162 => W'Val (  16#A2#), 163 => W'Val (  16#A3#), 164 => W'Val (  16#A4#),
      165 => W'Val (  16#A5#), 166 => W'Val (  16#A6#), 167 => W'Val (  16#A7#),
      168 => W'Val (  16#A8#), 169 => W'Val (  16#A9#), 170 => W'Val (  16#AA#),
      171 => W'Val (  16#AB#), 172 => W'Val (  16#AC#), 173 => W'Val (  16#AD#),
      174 => W'Val (  16#AE#), 175 => W'Val (  16#AF#), 176 => W'Val (  16#B0#),
      177 => W'Val (  16#B1#), 178 => W'Val (  16#B2#), 179 => W'Val (  16#B3#),
      180 => W'Val (  16#B4#), 181 => W'Val (  16#B5#), 182 => W'Val (  16#B6#),
      183 => W'Val (  16#B7#), 184 => W'Val (  16#B8#), 185 => W'Val (  16#B9#),
      186 => W'Val (  16#BA#), 187 => W'Val (  16#BB#), 188 => W'Val (  16#BC#),
      189 => W'Val (  16#BD#), 190 => W'Val (  16#BE#), 191 => W'Val (  16#BF#),
      192 => W'Val (  16#C0#), 193 => W'Val (  16#C1#), 194 => W'Val (  16#C2#),
      195 => W'Val ( 16#102#), 196 => W'Val (  16#C4#), 197 => W'Val (  16#C5#),
      198 => W'Val (  16#C6#), 199 => W'Val (  16#C7#), 200 => W'Val (  16#C8#),
      201 => W'Val (  16#C9#), 202 => W'Val (  16#CA#), 203 => W'Val (  16#CB#),
      204 => W'Val ( 16#300#), 205 => W'Val (  16#CD#), 206 => W'Val (  16#CE#),
      207 => W'Val (  16#CF#), 208 => W'Val ( 16#110#), 209 => W'Val (  16#D1#),
      210 => W'Val ( 16#309#), 211 => W'Val (  16#D3#), 212 => W'Val (  16#D4#),
      213 => W'Val ( 16#1A0#), 214 => W'Val (  16#D6#), 215 => W'Val (  16#D7#),
      216 => W'Val (  16#D8#), 217 => W'Val (  16#D9#), 218 => W'Val (  16#DA#),
      219 => W'Val (  16#DB#), 220 => W'Val (  16#DC#), 221 => W'Val ( 16#1AF#),
      222 => W'Val ( 16#303#), 223 => W'Val (  16#DF#), 224 => W'Val (  16#E0#),
      225 => W'Val (  16#E1#), 226 => W'Val (  16#E2#), 227 => W'Val ( 16#103#),
      228 => W'Val (  16#E4#), 229 => W'Val (  16#E5#), 230 => W'Val (  16#E6#),
      231 => W'Val (  16#E7#), 232 => W'Val (  16#E8#), 233 => W'Val (  16#E9#),
      234 => W'Val (  16#EA#), 235 => W'Val (  16#EB#), 236 => W'Val ( 16#301#),
      237 => W'Val (  16#ED#), 238 => W'Val (  16#EE#), 239 => W'Val (  16#EF#),
      240 => W'Val ( 16#111#), 241 => W'Val (  16#F1#), 242 => W'Val ( 16#323#),
      243 => W'Val (  16#F3#), 244 => W'Val (  16#F4#), 245 => W'Val ( 16#1A1#),
      246 => W'Val (  16#F6#), 247 => W'Val (  16#F7#), 248 => W'Val (  16#F8#),
      249 => W'Val (  16#F9#), 250 => W'Val (  16#FA#), 251 => W'Val (  16#FB#),
      252 => W'Val (  16#FC#), 253 => W'Val ( 16#1B0#), 254 => W'Val (16#20AB#),
      255 => W'Val (  16#FF#));

   KOI8_R_Table : aliased Translation_Table :=
     (128 => W'Val (16#2500#),
      129 => W'Val (16#2502#), 130 => W'Val (16#250C#), 131 => W'Val (16#2510#),
      132 => W'Val (16#2514#), 133 => W'Val (16#2518#), 134 => W'Val (16#251C#),
      135 => W'Val (16#2524#), 136 => W'Val (16#252C#), 137 => W'Val (16#2534#),
      138 => W'Val (16#253C#), 139 => W'Val (16#2580#), 140 => W'Val (16#2584#),
      141 => W'Val (16#2588#), 142 => W'Val (16#258C#), 143 => W'Val (16#2590#),
      144 => W'Val (16#2591#), 145 => W'Val (16#2592#), 146 => W'Val (16#2593#),
      147 => W'Val (16#2320#), 148 => W'Val (16#25A0#), 149 => W'Val (16#2219#),
      150 => W'Val (16#221A#), 151 => W'Val (16#2248#), 152 => W'Val (16#2264#),
      153 => W'Val (16#2265#), 154 => W'Val (  16#A0#), 155 => W'Val (16#2321#),
      156 => W'Val (  16#B0#), 157 => W'Val (  16#B2#), 158 => W'Val (  16#B7#),
      159 => W'Val (  16#F7#), 160 => W'Val (16#2550#), 161 => W'Val (16#2551#),
      162 => W'Val (16#2552#), 163 => W'Val ( 16#451#), 164 => W'Val (16#2553#),
      165 => W'Val (16#2554#), 166 => W'Val (16#2555#), 167 => W'Val (16#2556#),
      168 => W'Val (16#2557#), 169 => W'Val (16#2558#), 170 => W'Val (16#2559#),
      171 => W'Val (16#255A#), 172 => W'Val (16#255B#), 173 => W'Val (16#255C#),
      174 => W'Val (16#255D#), 175 => W'Val (16#255E#), 176 => W'Val (16#255F#),
      177 => W'Val (16#2560#), 178 => W'Val (16#2561#), 179 => W'Val ( 16#401#),
      180 => W'Val (16#2562#), 181 => W'Val (16#2563#), 182 => W'Val (16#2564#),
      183 => W'Val (16#2565#), 184 => W'Val (16#2566#), 185 => W'Val (16#2567#),
      186 => W'Val (16#2568#), 187 => W'Val (16#2569#), 188 => W'Val (16#256A#),
      189 => W'Val (16#256B#), 190 => W'Val (16#256C#), 191 => W'Val (  16#A9#),
      192 => W'Val ( 16#44E#), 193 => W'Val ( 16#430#), 194 => W'Val ( 16#431#),
      195 => W'Val ( 16#446#), 196 => W'Val ( 16#434#), 197 => W'Val ( 16#435#),
      198 => W'Val ( 16#444#), 199 => W'Val ( 16#433#), 200 => W'Val ( 16#445#),
      201 => W'Val ( 16#438#), 202 => W'Val ( 16#439#), 203 => W'Val ( 16#43A#),
      204 => W'Val ( 16#43B#), 205 => W'Val ( 16#43C#), 206 => W'Val ( 16#43D#),
      207 => W'Val ( 16#43E#), 208 => W'Val ( 16#43F#), 209 => W'Val ( 16#44F#),
      210 => W'Val ( 16#440#), 211 => W'Val ( 16#441#), 212 => W'Val ( 16#442#),
      213 => W'Val ( 16#443#), 214 => W'Val ( 16#436#), 215 => W'Val ( 16#432#),
      216 => W'Val ( 16#44C#), 217 => W'Val ( 16#44B#), 218 => W'Val ( 16#437#),
      219 => W'Val ( 16#448#), 220 => W'Val ( 16#44D#), 221 => W'Val ( 16#449#),
      222 => W'Val ( 16#447#), 223 => W'Val ( 16#44A#), 224 => W'Val ( 16#42E#),
      225 => W'Val ( 16#410#), 226 => W'Val ( 16#411#), 227 => W'Val ( 16#426#),
      228 => W'Val ( 16#414#), 229 => W'Val ( 16#415#), 230 => W'Val ( 16#424#),
      231 => W'Val ( 16#413#), 232 => W'Val ( 16#425#), 233 => W'Val ( 16#418#),
      234 => W'Val ( 16#419#), 235 => W'Val ( 16#41A#), 236 => W'Val ( 16#41B#),
      237 => W'Val ( 16#41C#), 238 => W'Val ( 16#41D#), 239 => W'Val ( 16#41E#),
      240 => W'Val ( 16#41F#), 241 => W'Val ( 16#42F#), 242 => W'Val ( 16#420#),
      243 => W'Val ( 16#421#), 244 => W'Val ( 16#422#), 245 => W'Val ( 16#423#),
      246 => W'Val ( 16#416#), 247 => W'Val ( 16#412#), 248 => W'Val ( 16#42C#),
      249 => W'Val ( 16#42B#), 250 => W'Val ( 16#417#), 251 => W'Val ( 16#428#),
      252 => W'Val ( 16#42D#), 253 => W'Val ( 16#429#), 254 => W'Val ( 16#427#),
      255 => W'Val ( 16#42A#));

   ATARIST_Table : aliased Translation_Table :=
     (128 => W'Val (  16#C7#),
      129 => W'Val (  16#FC#), 130 => W'Val (  16#E9#), 131 => W'Val (  16#E2#),
      132 => W'Val (  16#E4#), 133 => W'Val (  16#E0#), 134 => W'Val (  16#E5#),
      135 => W'Val (  16#E7#), 136 => W'Val (  16#EA#), 137 => W'Val (  16#EB#),
      138 => W'Val (  16#E8#), 139 => W'Val (  16#EF#), 140 => W'Val (  16#EE#),
      141 => W'Val (  16#EC#), 142 => W'Val (  16#C4#), 143 => W'Val (  16#C5#),
      144 => W'Val (  16#C9#), 145 => W'Val (  16#E6#), 146 => W'Val (  16#C6#),
      147 => W'Val (  16#F4#), 148 => W'Val (  16#F6#), 149 => W'Val (  16#F2#),
      150 => W'Val (  16#FB#), 151 => W'Val (  16#F9#), 152 => W'Val (  16#FF#),
      153 => W'Val (  16#D6#), 154 => W'Val (  16#DC#), 155 => W'Val (  16#A2#),
      156 => W'Val (  16#A3#), 157 => W'Val (  16#A5#), 158 => W'Val (  16#DF#),
      159 => W'Val ( 16#192#), 160 => W'Val (  16#E1#), 161 => W'Val (  16#ED#),
      162 => W'Val (  16#F3#), 163 => W'Val (  16#FA#), 164 => W'Val (  16#F1#),
      165 => W'Val (  16#D1#), 166 => W'Val (  16#AA#), 167 => W'Val (  16#BA#),
      168 => W'Val (  16#BF#), 169 => W'Val (16#2310#), 170 => W'Val (  16#AC#),
      171 => W'Val (  16#BD#), 172 => W'Val (  16#BC#), 173 => W'Val (  16#A1#),
      174 => W'Val (  16#AB#), 175 => W'Val (  16#BB#), 176 => W'Val (  16#E3#),
      177 => W'Val (  16#F5#), 178 => W'Val (  16#D8#), 179 => W'Val (  16#F8#),
      180 => W'Val ( 16#153#), 181 => W'Val ( 16#152#), 182 => W'Val (  16#C0#),
      183 => W'Val (  16#C3#), 184 => W'Val (  16#D5#), 185 => W'Val (  16#A8#),
      186 => W'Val (  16#B4#), 187 => W'Val (16#2020#), 188 => W'Val (  16#B6#),
      189 => W'Val (  16#A9#), 190 => W'Val (  16#AE#), 191 => W'Val (16#2122#),
      192 => W'Val ( 16#133#), 193 => W'Val ( 16#132#), 194 => W'Val ( 16#5D0#),
      195 => W'Val ( 16#5D1#), 196 => W'Val ( 16#5D2#), 197 => W'Val ( 16#5D3#),
      198 => W'Val ( 16#5D4#), 199 => W'Val ( 16#5D5#), 200 => W'Val ( 16#5D6#),
      201 => W'Val ( 16#5D7#), 202 => W'Val ( 16#5D8#), 203 => W'Val ( 16#5D9#),
      204 => W'Val ( 16#5DB#), 205 => W'Val ( 16#5DC#), 206 => W'Val ( 16#5DE#),
      207 => W'Val ( 16#5E0#), 208 => W'Val ( 16#5E1#), 209 => W'Val ( 16#5E2#),
      210 => W'Val ( 16#5E4#), 211 => W'Val ( 16#5E6#), 212 => W'Val ( 16#5E7#),
      213 => W'Val ( 16#5E8#), 214 => W'Val ( 16#5E9#), 215 => W'Val ( 16#5EA#),
      216 => W'Val ( 16#5DF#), 217 => W'Val ( 16#5DA#), 218 => W'Val ( 16#5DD#),
      219 => W'Val ( 16#5E3#), 220 => W'Val ( 16#5E5#), 221 => W'Val (  16#A7#),
      222 => W'Val (16#2227#), 223 => W'Val (16#221E#), 224 => W'Val ( 16#3B1#),
      225 => W'Val ( 16#3B2#), 226 => W'Val ( 16#393#), 227 => W'Val ( 16#3C0#),
      228 => W'Val ( 16#3A3#), 229 => W'Val ( 16#3C3#), 230 => W'Val (  16#B5#),
      231 => W'Val ( 16#3C4#), 232 => W'Val ( 16#3A6#), 233 => W'Val ( 16#398#),
      234 => W'Val ( 16#3A9#), 235 => W'Val ( 16#3B4#), 236 => W'Val (16#222E#),
      237 => W'Val ( 16#3C6#), 238 => W'Val (16#2208#), 239 => W'Val (16#2229#),
      240 => W'Val (16#2261#), 241 => W'Val (  16#B1#), 242 => W'Val (16#2265#),
      243 => W'Val (16#2264#), 244 => W'Val (16#2320#), 245 => W'Val (16#2321#),
      246 => W'Val (  16#F7#), 247 => W'Val (16#2248#), 248 => W'Val (  16#B0#),
      249 => W'Val (16#2219#), 250 => W'Val (  16#B7#), 251 => W'Val (16#221A#),
      252 => W'Val (16#207F#), 253 => W'Val (  16#B2#), 254 => W'Val (  16#B3#),
      255 => W'Val (  16#AF#));

   ISO_8859_1_Decoder : aliased Fixed_Decoder (ISO_8859_1_Table'Access);
   ISO_8859_2_Decoder : aliased Fixed_Decoder (ISO_8859_2_Table'Access);
   ISO_8859_3_Decoder : aliased Fixed_Decoder (ISO_8859_3_Table'Access);
   ISO_8859_4_Decoder : aliased Fixed_Decoder (ISO_8859_4_Table'Access);
   ISO_8859_5_Decoder : aliased Fixed_Decoder (ISO_8859_5_Table'Access);
   ISO_8859_6_Decoder : aliased Fixed_Decoder (ISO_8859_6_Table'Access);
   ISO_8859_7_Decoder : aliased Fixed_Decoder (ISO_8859_7_Table'Access);
   ISO_8859_8_Decoder : aliased Fixed_Decoder (ISO_8859_8_Table'Access);
   ISO_8859_9_Decoder : aliased Fixed_Decoder (ISO_8859_9_Table'Access);
   ISO_8859_10_Decoder : aliased Fixed_Decoder (ISO_8859_10_Table'Access);
   ISO_8859_11_Decoder : aliased Fixed_Decoder (ISO_8859_11_Table'Access);
   ISO_8859_13_Decoder : aliased Fixed_Decoder (ISO_8859_13_Table'Access);
   ISO_8859_14_Decoder : aliased Fixed_Decoder (ISO_8859_14_Table'Access);
   ISO_8859_15_Decoder : aliased Fixed_Decoder (ISO_8859_15_Table'Access);
   ISO_8859_16_Decoder : aliased Fixed_Decoder (ISO_8859_16_Table'Access);
   CP_037_Decoder : aliased Fixed_Decoder (CP_037_Table'Access);
   CP_424_Decoder : aliased Fixed_Decoder (CP_424_Table'Access);
   CP_437_Decoder : aliased Fixed_Decoder (CP_437_Table'Access);
   CP_500_Decoder : aliased Fixed_Decoder (CP_500_Table'Access);
   CP_875_Decoder : aliased Fixed_Decoder (CP_875_Table'Access);
   CP_737_Decoder : aliased Fixed_Decoder (CP_737_Table'Access);
   CP_775_Decoder : aliased Fixed_Decoder (CP_775_Table'Access);
   CP_850_Decoder : aliased Fixed_Decoder (CP_850_Table'Access);
   CP_852_Decoder : aliased Fixed_Decoder (CP_852_Table'Access);
   CP_855_Decoder : aliased Fixed_Decoder (CP_855_Table'Access);
   CP_856_Decoder : aliased Fixed_Decoder (CP_856_Table'Access);
   CP_857_Decoder : aliased Fixed_Decoder (CP_857_Table'Access);
   CP_860_Decoder : aliased Fixed_Decoder (CP_860_Table'Access);
   CP_861_Decoder : aliased Fixed_Decoder (CP_861_Table'Access);
   CP_862_Decoder : aliased Fixed_Decoder (CP_862_Table'Access);
   CP_863_Decoder : aliased Fixed_Decoder (CP_863_Table'Access);
   CP_864_Decoder : aliased Fixed_Decoder (CP_864_Table'Access);
   CP_865_Decoder : aliased Fixed_Decoder (CP_865_Table'Access);
   CP_866_Decoder : aliased Fixed_Decoder (CP_866_Table'Access);
   CP_869_Decoder : aliased Fixed_Decoder (CP_869_Table'Access);
   CP_874_Decoder : aliased Fixed_Decoder (CP_874_Table'Access);
   CP_1006_Decoder : aliased Fixed_Decoder (CP_1006_Table'Access);
   CP_1026_Decoder : aliased Fixed_Decoder (CP_1026_Table'Access);
   CP_1250_Decoder : aliased Fixed_Decoder (CP_1250_Table'Access);
   CP_1251_Decoder : aliased Fixed_Decoder (CP_1251_Table'Access);
   CP_1252_Decoder : aliased Fixed_Decoder (CP_1252_Table'Access);
   CP_1253_Decoder : aliased Fixed_Decoder (CP_1253_Table'Access);
   CP_1254_Decoder : aliased Fixed_Decoder (CP_1254_Table'Access);
   CP_1255_Decoder : aliased Fixed_Decoder (CP_1255_Table'Access);
   CP_1256_Decoder : aliased Fixed_Decoder (CP_1256_Table'Access);
   CP_1257_Decoder : aliased Fixed_Decoder (CP_1257_Table'Access);
   CP_1258_Decoder : aliased Fixed_Decoder (CP_1258_Table'Access);
   KOI8_R_Decoder : aliased Fixed_Decoder (KOI8_R_Table'Access);
   ATARIST_Decoder : aliased Fixed_Decoder (ATARIST_Table'Access);

   Table : constant array (Fixed_Width_Encoding) of Decoder_Access :=
     (ISO_8859_1 => ISO_8859_1_Decoder'Access,
      ISO_8859_2 => ISO_8859_2_Decoder'Access,
      ISO_8859_3 => ISO_8859_3_Decoder'Access,
      ISO_8859_4 => ISO_8859_4_Decoder'Access,
      ISO_8859_5 => ISO_8859_5_Decoder'Access,
      ISO_8859_6 => ISO_8859_6_Decoder'Access,
      ISO_8859_7 => ISO_8859_7_Decoder'Access,
      ISO_8859_8 => ISO_8859_8_Decoder'Access,
      ISO_8859_9 => ISO_8859_9_Decoder'Access,
      ISO_8859_10 => ISO_8859_10_Decoder'Access,
      ISO_8859_11 => ISO_8859_11_Decoder'Access,
      ISO_8859_13 => ISO_8859_13_Decoder'Access,
      ISO_8859_14 => ISO_8859_14_Decoder'Access,
      ISO_8859_15 => ISO_8859_15_Decoder'Access,
      ISO_8859_16 => ISO_8859_16_Decoder'Access,
      CP_037 => CP_037_Decoder'Access,
      CP_424 => CP_424_Decoder'Access,
      CP_437 => CP_437_Decoder'Access,
      CP_500 => CP_500_Decoder'Access,
      CP_875 => CP_875_Decoder'Access,
      CP_737 => CP_737_Decoder'Access,
      CP_775 => CP_775_Decoder'Access,
      CP_850 => CP_850_Decoder'Access,
      CP_852 => CP_852_Decoder'Access,
      CP_855 => CP_855_Decoder'Access,
      CP_856 => CP_856_Decoder'Access,
      CP_857 => CP_857_Decoder'Access,
      CP_860 => CP_860_Decoder'Access,
      CP_861 => CP_861_Decoder'Access,
      CP_862 => CP_862_Decoder'Access,
      CP_863 => CP_863_Decoder'Access,
      CP_864 => CP_864_Decoder'Access,
      CP_865 => CP_865_Decoder'Access,
      CP_866 => CP_866_Decoder'Access,
      CP_869 => CP_869_Decoder'Access,
      CP_874 => CP_874_Decoder'Access,
      CP_1006 => CP_1006_Decoder'Access,
      CP_1026 => CP_1026_Decoder'Access,
      CP_1250 => CP_1250_Decoder'Access,
      CP_1251 => CP_1251_Decoder'Access,
      CP_1252 => CP_1252_Decoder'Access,
      CP_1253 => CP_1253_Decoder'Access,
      CP_1254 => CP_1254_Decoder'Access,
      CP_1255 => CP_1255_Decoder'Access,
      CP_1256 => CP_1256_Decoder'Access,
      CP_1257 => CP_1257_Decoder'Access,
      CP_1258 => CP_1258_Decoder'Access,
      KOI8_R => KOI8_R_Decoder'Access,
      ATARIST => ATARIST_Decoder'Access);

   UTF_8 : aliased Gela.Decoders.UTF_8.Decoder;

end Gela.Decoders.Constants;


------------------------------------------------------------------------------
--  Copyright (c) 2006-2013, Maxim Reznik
--  All rights reserved.
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are met:
--
--     * Redistributions of source code must retain the above copyright notice,
--       this list of conditions and the following disclaimer.
--     * Redistributions in binary form must reproduce the above copyright
--       notice, this list of conditions and the following disclaimer in the
--       documentation and/or other materials provided with the distribution.
--     * Neither the name of the Maxim Reznik, IE nor the names of its
--       contributors may be used to endorse or promote products derived from
--       this software without specific prior written permission.
--
--  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
--  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
--  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
--  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
--  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
--  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
--  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
--  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
--  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
--  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.
------------------------------------------------------------------------------
